#! /usr/bin/env python
""" Script to apply the results of a previously run dance to a new kernel.
New PSF files will be created, as will a new "dancecard" file which can
be used in another pass through TFIT."""

from tfit.tfit_dance import PSF
from tfit import sextutils
from pyraf import iraf
from iraf import imlintran
import re, sys



def main(psffile, ddiags):
    psf=PSF(psffile)
    zones=sextutils.se_catalog(ddiags)
    shiftlog=open(ddiags+'_applied_shiftlog','w')
    newcard=ddiags+'_applied_dancecard'
    out=open(newcard,'w')
    ix = -1
    iy = -1
    old_cx=-1
    for k in zones:
        iy+=1
        if zones.cx[k] != old_cx:
            ix+=1
            iy=0
            old_cx = zones.cx[k]
        lx,ux,ly,uy=parserange(zones.range[k])
        if zones.xshift[k] != 0 or zones.yshift[k] != 0:
            psfname = psf.name.replace(".fits","_%02d_%02d.fits"%(ix,iy))
            iraf.imlintran(xin=psf.xcent,
                           yin=psf.ycent,
                           xout=psf.xcent + zones.xshift[k],
                           yout=psf.ycent + zones.yshift[k],
                           input=psf.name,
                           output=psfname,
                           xmag=1.0, ymag=1.0,
                           xrot=0.0,yrot=0.0,
                           ncols='INDEF', nlines='INDEF',
                           interp='drizzle',
                           bound='constant',
                           constant=0.0,
                           mode='h',
                           Stdout=shiftlog)
        else:
            psfname=psf.name
        out.write("%d  %d  %d  %d  %s\n"%(lx,ux,ly,uy,psfname))
    out.close()
    shiftlog.close()
            
    
            
    
def parserange(rangestring):
    tmp=re.split(r'[:,\[\]]',rangestring)
    lx,ux,ly,uy=tmp[1:-1]
    return int(lx),int(ux),int(ly),int(uy)

if __name__ == '__main__':
    if '-h' in sys.argv or len(sys.argv) < 3:
        print "Usage: applydance.py <name of new psf file> <ddiags filename>"
    else:
        psfname, ddiags = sys.argv[1:3]
        main(psfname,ddiags)
