#!/usr/bin/env python

def main(cellname):
    """ Convert a TFIT .cell file to a DS9 region file, so the cells can be
    overplotted onto the image"""
    #Open the cell file
    f=open(cellname)
    #Open the region file
    out=open(cellname.replace(".cell",".reg"),'w')
    out.write('# Region file format: DS9 version 3.0\n')
    out.write('global color=green font="helvetica 10 normal" select=1 edit=1 move=1 delete=1 include=1 fixed=0 source\n')

    #Set up the color dictionary
    color={0:'red',1:'green',2:'blue',3:'yellow'}
    k=0
    #Start the loop
    for line in f:
        if not line.startswith('#'):
            k+=1
            cols=line.split()
            nx=float(cols[2])
            ny=float(cols[3])
            lx=float(cols[4])
            ly=float(cols[5])
            cx,cy=centercalc(lx,ly,nx,ny)
            out.write("image;box(%f,%f,%f,%f,0) # color = %s\n"
                      %(cx,cy,nx,ny,color[k%4]))
    f.close()
    out.close()

def centercalc(lx,ly,nx,ny):
    """ Compute cell center from lower left corner & dimensions"""
    cx=lx+nx/2
    cy=ly+ny/2
    return cx,cy

if __name__ == '__main__':
    import sys
    cellfile=sys.argv[1]
    main(cellfile)
                      
