import os, copy
from tfit import template, fitsimage

def transform(LoResFile,HiResFile,catfile,newcatfile):
    """ Transform the (unculled) input tfit catalog to be in the lores reference frame. """

    assert os.path.exists(LoResFile), "Cull: LoResFile %s does not exist"%LoResFile
    assert os.path.exists(HiResFile), "Cull: HiResFile %s does not exist"%HiResFile
    assert os.path.exists(catfile), "Cull: catfile %s does not exist"%catfile


    #Get the lores header
    LoRes = fitsimage.Image(LoResFile)
    LoRes.wcs.fillcd()

    #Get the hires header
    #... modification to permit multi-tile catalogs
    if HiResFile.lower().endswith('fits'):
      hwcs = fitsimage.Image(HiResFile).wcs
      hwcs.fillcd()
    else: #it's a text file. Read its info.
      hf=open(HiResFile).readlines()
      wdict={}
      for line in hf:
        key,value=line.split()
        wdict[key.upper()]=float(value)
      #Start with the lores wcs
      hwcs=copy.deepcopy(LoRes.wcs)

      #set the scaling
      hwcs.rescale(-1*wdict['REL_SCALE'])

      #Replace the crpixes
      hwcs.set_crpix(wdict['CRPIX1'],wdict['CRPIX2'])

                     
    #Read the catalog, transform, & write out again
    newcat=open(newcatfile,'w')
    oldcat=open(catfile,'r')
    for line in oldcat:
	if not line.startswith('#'):
	    id,x,y,rest = line.split(None,3)
            nx,ny = LoRes.wcs.xy2xy(float(x),float(y),hwcs)
            nline=' '.join([id,`float(nx)`,`float(ny)`,rest.rstrip(),"\n"])
            newcat.write(nline)      


def cull(LoResFile,catfile,indir,newcatfile):
    """ cull the templates, but work from the already-transformed catalog.
    This is a substantial change from the previous incarnation tlibrary.cull. """
    
    assert os.path.isdir(indir), "Cull: Input directory %s is not a directory"%indir
    assert os.path.exists(LoResFile), "Cull: LoResFile %s does not exist"%LoResFile
    assert os.path.exists(catfile), "Cull: catfile %s does not exist"%catfile

    #TTD: insert use of lores weightmap, to set weight_has_good_data

    #Get the lores header
    LoRes = fitsimage.Image(LoResFile)
    LoRes.wcs.fillcd()
                     
    weight_has_good_data = 1  #Insert WtMapFile,WtMap later

    #Read the catalog.
    #Note that it has already been transformed to lores coords
    oldcat=open(catfile,'r').readlines()

    #Start some bookkeeping lists
    newcat = open(newcatfile,'w')
    ncount=0
    rmlist = []
    plist = []

    

    #Go through each object & determine whether to keep as is, change, or cull
    #Do the bookkeeping and take the appropriate actions for each object.
    for line in oldcat:
	if not line.startswith('#'):
	    eliminate=False
	    change = False
	    id,x,y,rest = line.split(None,3)
	    t=indir+'/'+id+'.fits'
            
            try:
              thisone = fitsimage.Image(t)
              if LoRes.wcs_box.Contains(thisone.wcs_box):
                  pass
              else:
                  if (LoRes.wcs_box.Overlaps(thisone.wcs_box) or
                      thisone.wcs_box.Overlaps(LoRes.wcs_box)): #can happen!
                  
                      print "Overlap detected: id ",id
                     # print thisone.wcs_box
                      thisone.ShrinksToFit(LoRes)
                      plist.append(os.path.basename(t))
                      change=True
                  else:
                      eliminate=True

      ## 	sub = WtMap.xypos(thisone)
      ## 	weight = WtMap.d[sub.Slice()]

              if (weight_has_good_data):
                  if change:
                      newname=os.path.dirname(t)+'/partial_'+os.path.basename(t)
                      thisone.writeto(newname)
              else:
                  eliminate=True

              if not eliminate:
                  #Push the object center into the lores range
                  #Flag the objects for which this moves them.
                  nx, xf = template.clip(LoRes.local_box.Xrange(),float(x)) 
                  ny, yf = template.clip(LoRes.local_box.Yrange(),float(y)) 
                  nline="%s  %f  %f %s  %d\n"%(id,float(nx),float(ny),rest.rstrip(),xf+yf)
                  newcat.write(nline)
                  ncount+=1
              else:
                  rmlist.append(t)
                  
            except IOError: #Permits using a catalog containing objects not cutout
              rmlist.append(t)
    
    #Tell us the final counts
    print "Size of catalog to keep  : ",ncount
    print "Number of partial files  : ",len(plist)
    print "Number of files to remove: ",len(rmlist)


    #Now clean up the "partial" files
    if len(plist) > 0:
      try:
	os.mkdir(indir+'/shrunk')
      except OSError,e:
        if e.strerror == 'File exists':
          pass
        else:
          raise e

      for thing in plist:
          oldname=indir+'/'+thing
          os.rename(oldname,indir+'/shrunk/full_'+thing)
          os.rename(indir+'/partial_'+thing,indir+'/'+thing)

    #and "remove" the fits files
    if len(rmlist) > 0:
      try:
	os.mkdir(indir+'/culled')
      except OSError,e:
        if e.strerror == 'File exists':
          pass
        else:
          raise e

      for thing in rmlist:
          if os.path.isfile(thing):
              newname=os.path.dirname(thing)+'/culled/'+os.path.basename(thing)
              os.rename(thing,newname)

    #return the status info
    return "Number kept/partial/removed: %d/%d/%d"%(ncount,len(plist),len(rmlist))
