"""Utility functions for working with a sextutils.se_catalog object
that defines a grid.
  Required columns are [lx,ux,ly,uy]: these define a Box which is used
as a key to the grid.
  At least one of [kernel | (xshift, yshift)] must also be specified."""
import box

def findbin(lookup,x,y):
  """ Find the bin containing an object. """
  for k in lookup:
    if k.Includes(x,y):
      return(lookup[k])
  #If (x,y) is contained in no bins, we'll fall through to here
  raise IndexError('No bin containing (%f,%f) found in lookup table'%(x,y))


def parsegrid(gridcat,target_column):
  d={}
  for k in gridcat:
    key=box.Box(x1=gridcat.lx[k],y1=gridcat.ly[k],
                x2=gridcat.ux[k],y2=gridcat.uy[k])
    d[key]=gridcat.__getattribute__(target_column)[k]

  return d

def verify_grid(gridcat):
    """Enforce the rules for legal gridfiles:
    - if one of (xshift,yshift) is present, both must be.
    - one of (kernel, xshift, yshift) must be present (or it's a no-op) """
    xshift=hasattr(gridcat,'xshift')
    yshift=hasattr(gridcat,'yshift')
    kernel=hasattr(gridcat,'kernel')

    if (xshift and not yshift) or (yshift and not xshift):
        raise ValueError('%s invalid: both xshift and yshift must be specified'%gridcat._fname)

    if not xshift and not kernel:
        raise ValueError("%s invalid: at least one of [kernel|(xshift and yshift)] must be specified"%gridcat._fname)
