#!/usr/bin/env/python
# 
# Version 1.0.0 -- 9/25/03
# Version 1.0.1 -- 4/09/04  -- fixed spelling of booleans
# Version 1.0.2 -- 7/15/04  -- Ignore blank lines
#
""" Routines for parsing SExtractor-style input files.

__author__ =  'Henry C. Ferguson'
__version__=  '1.0.3'
""" 

 
import string

def main(configfile,paramdict={},preservecase=False):
  """ Try a more pythonic version? """

  f=open(configfile)
  for line in f:
    line=line.strip() #trim whitespace
    if not line.startswith('#') and len(line) > 0:
      #strip out trailing comments, if any
      try:
        ll,junk = line.split('#',1)
      except ValueError:
        ll=line
      #replace commas with spaces, then split on spaces
      ll=ll.replace(',',' ')
      keyword,vals=ll.split(None,1)
      if not preservecase:
        keyword=keyword.lower()
      #Do we have multiple values?
      if len(vals.split()) == 1:
        paramdict[keyword] = truval(vals)
      else:
        paramdict[keyword] = []
        for x in vals.split():
          paramdict[keyword].append(truval(x))
  return paramdict

def truval(x):
  """ Return the properly typed value of x """
  boolvals={'true':True, 'false':False, 't': True, 'f': False,
            'yes': True, 'no': False, 'y':True, 'n': False}
  x=x.strip() #strip off leading & trailing whitespace first
  try: #is it an int?
    ans=int(x)
  except ValueError:
    try: #is it a float?
      ans = float(x)
    except ValueError: #is it a literal?
      if x.startswith("'") or x.startswith('"'):
        ans=x[1:-1]
      else:
        try: #is it a boolean?
          ans=boolvals[x.lower()]
        except: #must be a string
          ans=x

  return ans

def parseconfig(configfile,paramdict={},preservecase=False):
  """ synonym for main, for backwards-compatibility """
  return main(configfile,paramdict=paramdict,preservecase=preservecase)
