""" Classes and a factory function to facilitate constructing various
TFIT pipelines."""

__version__ = 0.1

class Stage:
    """ Parent class for a processing stage """

    def __init__(self,name,exestring,*params,**kwds):
        self.name=name
        self.exestring=exestring
        self.params=params
        self.keywords=kwds
        self.status=None
        self.retvalue=None

    def __str__(self):
        return self.name
    

    def run(self):
        print "Running %s"%self.name
        self.retvalue = self.exestring(*self.params,**self.keywords)
        self.status=0

#..............................................................
class Pipeline(list):
    def __init__(self,name,catch=False):
        self.name=name
        self.catch=catch
        
    def run(self):
        if self.catch:
            self.run_catchall()
        else:
            for stage in self:
                stage.run()

    def run_catchall(self):
        for stage in self:
            try:
                stage.run()
            except Exception, e:
                print '\t',e,': Proceeding to next stage'
                
    def show(self):
        d={True:'continue',False:'stop'}
        header='%s: Processing will %s if a task raises an exception '
        print header%(self.name, d[self.catch])
        for stage in self:
            print "\t",stage


    def status(self):
        for stage in self:
            print "%s: %s"%(stage.name,stage.status)

    def answers(self):
        print "\n %s results:"%self.name
        for stage in self:
            print "\t%s: %s"%(stage.name,stage.retvalue)
        print "\n"
#..............................................................

def test():
    import parseconfig
    import tfit_stages
    
    d=parseconfig.main('test_tpipe.param',{})
    pipe=Pipeline('tpipe.param')
    for sname in d['order']:
        pipe.append(tfit_stages.StageFactory(sname,d))
    pipe.show()
    pipe.run()
    pipe.show()
    pipe.answers()


