
#! /usr/bin/env python

from pylab import *
from tfit import sextutils
import sys,os
import numpy as N

def main(dancefile,scale=10):
    """ Read a file containing x,y,dx,dy & make a vector plot """
    #Save the interactivity setting, then turn off interaction
    isetting=isinteractive()
    ioff()
    
    zones=sextutils.se_catalog(dancefile)
    zhandle,shandle=vectorplot(zones.cx,zones.cy,
                               zones.xshift,zones.yshift,scale=10)
    
    #overplot the interpolated points
    idx=N.where(zones.interpolated == 1)
    ihandle=plot(zones.cx[idx],zones.cy[idx],'rv',ms=3)
    
    #add some labels
    title(dancefile)
    legend([shandle,zhandle,ihandle],
           ['shifts [* %4.1f]'%scale,'zone centers','interpolated'])

    #save
    plotname=os.path.splitext(dancefile)[0]+'.ps'
    savefig(plotname)

    #Restore interactivity setting
    matplotlib.interactive(isetting)
    
    return plotname

def vectorplot(x,y,dx,dy,scale=1):
    """ Generate a vector plot with lines emanating from the (x,y)
    points that represent the (dx,dy) displacement at each point.

    @param scale: Multiply (dx,dy) by this factor to make the displacements
    more visible

    @return: (cell_handle, displacement_handle) which can be used
    to pass to "legend"
    """

    u=x+scale*dx
    v=y+scale*dy

    #loop through to make the plots
    for i in range(len(x)):
        dhandle=plot([x[i],u[i]],[y[i],v[i]],'-b',
                     label='shifts [* %4.1f]'%scale)

    #then overplot the origin points in a different color
    chandle=plot(x,y,'.k',label='zone',ms=0.8)

    return (chandle,dhandle)

if __name__ == '__main__':
    dancefile=sys.argv[1]
    if len(sys.argv)==3:
        main(dancefile,scale=scale)
    else:
        main(dancefile)
