#! /usr/bin/env python
from tfit import tfit_diags,fitsimage, futil, sextutils
import sys


def main(cutoutdir,hrtdir,hiresfile,bestfile=None):
    """ Make a collage. If a bestfile is supplied, drive the collage from
    it, and apply the scale factor it contains. If not, drive the collage
    from the template directory. """
    if bestfile:
        sf=readbest(bestfile)
    hires=fitsimage.Image(hiresfile)
    #First make the cutout collage
    coll=tfit_diags.collage(hires.d.shape,cutoutdir,bigwcs=hires.wcs,mult=sf)
    futil.writeimage(coll,'hrpsf_collage.fits',header=hires.h)


    #Then make the hr template collage
    coll=tfit_diags.collage(hires.d.shape,hrtdir,bigwcs=hires.wcs,mult=sf)
    futil.writeimage(coll,'lrpsf_collage.fits',header=hires.h)
                

def readbest(bestfile):
    """ read the scale factor from the already-created bestfile"""
    cat=sextutils.se_catalog(bestfile)
    d={}
    for k in cat:
        d[str(cat.objectid[k])]=cat.fitqty[k]
    return d

if __name__ == '__main__':
    try:
        cutoutdir,hrtdir,hiresfile,bestfile=sys.argv[1:]
        main(cutoutdir,hrtdir,hiresfile,bestfile)
    except ValueError:
        print "Usage: psf_collage.py cutoutdir hrtdir hiresfile bestfile"
        

