#! /usr/bin/env python
#
#On old goods cluster, I need this:
#!/usr/stsci/pyssg/Python-default/bin/python

#On linux cluster, I need this:
#!/usr/stsci/pyssgdev/Python-2.4.1/bin/python
#
import glob,os,time,shutil,sys,time, random
import tfit_pipeline, parseconfig  
from pyraf import iraf
import numarray as N
import pyfits
import simtfit_util
import rms2noise
import readcol

__version__="0.0.8"

#History
#  0.0.1 - spun off from run_sim_tfit.py v0.0.6 + nopsf changes
#  0.0.2 - delete the intermediate image products to save space
#  0.0.3 - Permit generating fittarg image by convolving master image
#  0.0.31  Bugfix: adjust for zeropoint difference between images
#  0.0.32  Bugfix to the bugfix: I got the sign wrong 
#  0.0.4   New implementation of from_master method
#  0.0.41  Ahem: forgot to convolve the lores image with its psf. oops!
#  0.0.42  Support different levels of file cleanup through a new param, DELLEVEL.
#          If parameter is missing, default "normal" value is used.
#  0.0.5   Use the targimage.psffile when generating the target image from the
#          master image; not the tkernel file. Necessary for mismatch-psf tests.
#  0.0.6   Allow adding some randomly generated noise to the object positions,
#          to simulate misregistration.
#  0.0.7   Allow adding Poisson noise to the objects: this involved rewriting
#          master.addgals and fittarg.from_master to call mkobjects directly
#          instead of through the rsm.makenoiselessimage routine; and adding
#          a parameter to the sim file.
#          Also, cleaned up old (unused) from_master routine & rename
#          from_master2 to from_master.
#  0.0.8   Reworked to work with TFIT 1.0beta user interface; also stripped
#          functions out of run_sim_multi & put then in a special util file
#
#Changes to make:
#  - zip up the iter files (or iter dirs)
#  - actually generate some diagnostics (none presently)
#  - remove scale from the .sim file & get it from the header files (for safety!!!)

def main(argfile='simtfit.dat',setup=None):
    """ Start out considering pairs of images only. """
    #Tell us who you are
    print "Running %s version %s from file \n%s"%(__name__,__version__,__file__)
    
    #Read the configuration file
    c=parseconfig.main(argfile,preservecase=True)
    c['ROOT']=argfile[:-4]
    artpars=SimGalSet(c)


    #set the verbose setting
    verbose=True
    if not c['VERBOSE']:
        verbose=False


    #Create an object for the master (hi res) image &
    # the fit target (lores) image
    master=Masterimage(c['BANDS'][0],c,verbose=verbose)
    fittarg=Targimage(c['BANDS'][1],c,verbose=verbose)

    #Create an object for all the tfit parameters
    tfitpars=TfitPars(c,master,fittarg)
    
    #Now start the iterations.
    for n in range(1,c['NITER']+1):
        print "\n            ------------------------Iteration %d---------------------------\n"%n
        #Regenerate the substrate images for each iteration
        master.gen_substrate()
        fittarg.gen_substrate()

        #Generate the list of objects & create the hires image
        master.gengals(artpars)
        master.addgals(verbose=verbose,srcnoise=c.get('SRCNOISE','no'))
        
        #Generate the lores image either from artdata (in which
        #case we need to transform the object list) or simply by
        #convolving the hires image

        if c['LORES_METHOD'].lower() == 'master':
            fittarg.colorgals(master,color=c.get('COLOR',None),
                              colorsig=c.get('COLORSIG',None),
                              maxshift=c.get('MAXSHIFT',None))
            fittarg.from_master(master, tfitpars, verbose=verbose,
                                srcnoise=c.get('SRCNOISE','no'))
        else:
            fittarg.movegals(master,
                             color=c.get('COLOR',None),
                             colorsig=c.get('COLORSIG',None)) 
            fittarg.addgals(verbose=verbose,srcnoise=c.get('SRCNOISE','no'))


        #SExtract both images
        master.sextract()
        fittarg.sextract()

        #Run TFIT
        tfit_pipeline.run(tfitpars.tparname)

        #Cleanup
        cleanup(n,master,fittarg,tfitpars)

def cleanup(n,master,fittarg,tfitpars):
        outdir='iter%s'%n
        os.mkdir(outdir)
        master.movefiles(outdir)
        fittarg.movefiles(outdir)
        tfitpars.movefiles(outdir)


    
class TfitPars:
    """Create all the info ro run the TFIT pipeline """

    def __init__(self,c,master,fittarg):

        self.tparname='tfit_pipeline.param'
        self.tpar={}
        
        #Set up some default values that don't change

        self.tpar['errtype']='rms'
        self.tpar['rmsconstant']=1.0
        self.tpar['bgconstant']=0.0
        self.tpar['order']='cutout, convolve, transform, cull, fit, diags'
        self.tpar['hicatcolnames']=None
        self.tpar['dithercell']=True
        self.tpar['multikernels']=False


        #Set up the default filenames
        self.tpar['cutoutdir']='cutouts/'
        self.tpar['templatedir']='templates/'
        self.tpar['modelfile']='collage.fits'
        self.tpar['templatecat']='templates.cat'
        self.tpar['tfitpars']='fitter.param'
        
        #Then add the filenames that derive from the objects
        self.tpar['hiresfile']=master.outimage
        self.tpar['hiresseg']=master.segfile
        self.tpar['hirescat']=master.scat
        self.tpar['relscale']=int(float(fittarg.scale/master.scale))
        self.tpar['loresfile']=fittarg.outimage
        self.tpar['cutoutcat']=master.scat+'_hrc_trim.txt'
        self.tpar['lrfcat']=master.scat+'_lrf.txt'
        


        self.tpar['loresfile']=fittarg.outimage
        self.tpar['loreserr']=fittarg.rmsimage
        self.tpar['tfitcat']=fittarg.tcat
        self.tpar['tfitcell']=fittarg.cellfile
        self.tpar['tfitcovar']=fittarg.covar



        #Add the values that are in the sim file
        self.tpar['normalize']=c['NORMALIZE']
        self.tpar['psffile']=c['T_KERNEL']
        self.tpar['cell_xdim']=c['XBUFFERSIZE']
        self.tpar['cell_ydim']=c['YBUFFERSIZE']
        self.tpar['cell_overlap']=c['CELL_OVERLAP']

        self.tpar['loresflag']=c['FLAG_MAP']
        self.tpar['maxflag']=c['MAX_FLAG']
        self.tpar['cellmask']=c['CELLMASK']
        self.tpar['maskfloor']=c['FLOOR']
        self.tpar['writecovar']=c['EXPORT_COVAR_INFO']
        self.tpar['fitbackground']=c['FIT_BACKGROUND']
        
 

        
        #For bookkeeping, make the list of files to cleanup
        self.files_to_clean=[self.tpar['cutoutdir'],
                             self.tpar['templatedir'],
                             self.tpar['templatecat'],
                             self.tpar['modelfile'],
                             'resid_'+self.tpar['modelfile'],
                             self.tpar['tfitpars'],
                             self.tpar['tfitpars'].replace('.param','.log')
                             ]

        #Finally, create the new parameter file
        self.create_parfile(self.tpar,self.tparname)

    def movefiles(self,outdir):
        for fname in self.files_to_clean:
            os.rename(fname,outdir+'/'+fname)

    def create_parfile(self, pardict, filename):
        """ Create a tfit-style parfile from a dictionary """

        out=open(filename,'w')
        out.write("#This file autocreated by %s \n"%__name__)
        out.write("#  at %s\n"%time.ctime())
        out.write("# \n")
        for k in pardict:
            out.write("%s     %s\n"%(k,pardict[k]))
        out.close()

class SimGalSet:
    """ Create all the info that artdata will need to put in the
    simulated galaxies."""
    
    def __init__(self,c):
#        self.root= c['ROOT']
        self.rmin= c['RMIN']   #/c['SCALE']
        self.rmax= c['RMAX']   #/c['SCALE']
        self.rdistpars=[]
        self.rdistfunc=c['RADIUS_DISTRIBUTION']
        if self.rdistfunc == 'lognormal':
            self.rdist_sigma = c['LOGNORMAL_SIGMA']
            self.rdist_r0    = c['LOGNORMAL_PEAK']
            self.rdist_mag0  = c['LOGNORMAL_MAG0']
            self.rdist_beta  = c['LOGNORMAL_BETA']

        self.ngal=c['NGALAXIES']
        self.diskfrac=c['DISKFRAC']
        self.maglow=c['MAGLOW']
        self.maghigh=c['MAGHIGH']
        self.edgebuffer=c['EDGEBUFFER']
        self.circularflag=c['CIRCULAR']
        self.flagmin=c['FLAGMIN']

        #Custom magnitude distribution:
        if c.get('MAGDIST_FILE',None) is not None:
            #Read the file
            f=open(c['MAGDIST_FILE'])
            magdist=[]
            for line in f:
                magdist.append(float(line.strip()))
            f.close()
            mdist=N.array(magdist)
            #Trim it to be between the limits
            idx=((mdist >= c['MAGLOW']) & (mdist <= c['MAGHIGH']))
            self.magdist=mdist[idx]
            #Shuffle it to be sure it's not a sorted list
            random.shuffle(self.magdist)

   

class Photimage:
    def __init__(self,band,c,verbose=0):
        band_idx=c['BANDS'].index(band)
        self.band=band
        
        #Properties of the image
        self.magz=c['MAGZPT'][band_idx]
        self.scale=c['SCALE'][band_idx]
        self.hdrfile=c['HDRFILE'][band_idx]
        
        #Creating the simulated data prior to adding galaxies

        self.rmsval=c['RMSVAL'][band_idx]
        self.xdim=self.ydim=c['IMDIMS'][band_idx]
        #Don't actually create this image at creation; we'll redo it in each iter.
        self.substrate=band+'_substrate.fits'
        
        #Create the rms and flag image, used by SExtractor

        self.rmsimage=band+'_rms.fits'
        rms2noise.writeconstrms(self.rmsval,(self.xdim,self.ydim),
                                 self.hdrfile,self.rmsimage)

        self.flagimage=band+'_flag.fits'
        d=N.zeros( (self.xdim,self.ydim),N.typecode['Int32'])
        f=pyfits.open(self.hdrfile)
        f[0].data=d
        f.writeto(self.flagimage)

        self.outimage=band+'_outimage.fits'
        
        #The PSF file (special case for "none")
        self.psffile=c['PSFFILE'][band_idx]
        if self.psffile.upper()=='NONE': self.psffile=''

        #The SExtractor parameters
        self.sexconfigfile=c['SEXFILE'][band_idx]
        self.sextparfile=c['SEXTPARFILE'][band_idx]
        self.gain=c['GAIN'][band_idx]
        self.scat=band+'_bsx.cat'
        self.segfile=band+'_seg.fits'
        
        #The files pertaining to adding the galaxies
        self.artdata_file=band+'_artdata.dat'
        self.artimage=band+'_artimage.fits'


        #Do some bookkeeping to make cleanup easier
        self.dellevel=c.get('DELLEVEL','normal').lower()
        self.files_to_clean=[self.outimage,
                             self.scat, 
                             self.artdata_file]
        
        self.files_to_delete=[self.substrate]
        if self.dellevel.startswith('min'):
            self.files_to_clean+=[self.artimage,self.segfile]
        elif self.dellevel.startswith('norm'):
            self.files_to_clean+=[self.segfile]
            self.files_to_delete+=[self.artimage]
        else:
            self.files_to_delete+=[self.artimage,self.segfile]
            

    def readgals(self):
        """ Read in the galaxy list so we have them """
        f=readcol.readcol(self.artdata_file)
        self.artx,self.arty,self.artmag=f.getcols(1,2,3)
        f.close()
        
    def artid(self):
        """ Identify the objects in the artcat by using the segfile """
        self.readgals()
        seg=pyfits.open(self.segfile)[0].data
        self.artid={}
        self.blendcount=0
        for i in range(len(self.artx)):
            id=seg[int(self.arty[i])-1,int(self.artx[i])-1]
            if self.artid.has_key(id):
                #It's a duplicate; set a flag
                self.artid[id]=-99
                self.blendcount+=1
            else:
                self.artid[id]=self.artmag[i]
            
    def movefiles(self,outdir):
        """ Move all the generated files into the output directory """
        for fname in self.files_to_clean:
            os.rename(fname,outdir+'/'+fname)
        if self.dellevel != 'none':
            for fname in self.files_to_delete:
                os.remove(fname)
                 
    def gen_substrate(self,verbose=True):
        """ Generate the substrate image, ie, with the random bg noise """
        if verbose:
            print "Creating %s data files..."%self.band

        rms2noise.writenoise(0.0,self.rmsval,(self.xdim,self.ydim),
                             self.hdrfile,self.substrate)

    def addgals(self,verbose=0,srcnoise="no"):
        print "srcnoise=%s"%srcnoise
        if verbose:
            print "Adding galaxies to %s using PSF %s"%(self.substrate,self.psffile)
        #Simulate the objects
        iraf.artdata.dynrange=1.e5
        iraf.mkobjects(self.artimage,
                       output="",
                       title="",
                       ncols=self.xdim,
                       nlines=self.ydim,
                       header=self.hdrfile,
                       background=0.0,
                       objects=self.artdata_file,
                       xoffset=0.0,
                       yoffset=0.0,
                       star="gaussian",
                       radius=0.1,
                       beta=2.5,
                       ar=1.0,
                       pa=0.0,
                       distance=1.0,
                       exptime=1.0,
                       magzero=self.magz,
                       gain=self.gain,
                       rdnoise=0.0,
                       poisson=srcnoise,
                       seed=2,
                       comments=1)

        #convolve with PSF
        if self.psffile:
            simtfit_util.convolve(os.path.splitext(self.artimage)[0],
                         self.psffile,
                         save=1) #always save; clean up later
        #add background
        iraf.imcalc(self.artimage+","+self.substrate,self.outimage,"im1+im2")
        
 ##            rsm.makenoiselessimage(self.artdata_file,
##                             os.path.splitext(self.artimage)[0],
##                             self.magz,
##                             self.xdim,
##                             self.ydim,
##                             gain=self.gain, 
##                             psffile=self.psffile,
##                             hdrfile=self.hdrfile,
##                             save=1) #always save


        
    def sextract(self):
        """Construct the command line & run SExtractor"""
        s_args = "%s -c %s" %(self.outimage,self.sexconfigfile)

        s_args = s_args + " -CATALOG_NAME %s" % (self.scat)
        s_args = s_args + " -MAG_ZEROPOINT %9.4f" % self.magz
        s_args = s_args + " -GAIN %12.4f" % self.gain
        s_args = s_args + " -FLAG_IMAGE %s" % self.flagimage
        s_args = s_args + " -PARAMETERS_NAME %s" % self.sextparfile
        s_args = s_args + " -CHECKIMAGE_TYPE SEGMENTATION"
        s_args = s_args + " -CHECKIMAGE_NAME %s" % (self.segfile)
        s_args = s_args + " -WEIGHT_TYPE MAP_RMS"
        s_args = s_args + " -WEIGHT_IMAGE %s" % (self.rmsimage)
        print "sex %s" % (s_args)
        sys.stdout.flush()
        fpipe = os.popen("sex %s" % (s_args))
        fpipe.close()        
        
class Targimage(Photimage):
    def __init__(self,band,c,verbose=0):
        Photimage.__init__(self,band,c,verbose=verbose)
        self.tcat=band+'_tfit.cat'
        self.cellfile=band+'_tfit.cell'
        self.covar=band+'_covar.dat'

        
        self.files_to_clean+=[self.tcat, self.tcat+'_best', self.cellfile, self.covar]

        if c.get('LORES_METHOD','artdata') == 'artdata':
            self.files_to_delete+=[self.artimage.replace('.fits','_nopsf.fits')]
        else:
            self.hiresart=self.artimage.replace('artimage','artimage_hr')
            self.hirescnv=self.hiresart.replace('artimage','artimcnv')
            if self.dellevel.startswith('min'):
                self.files_to_clean+=[self.hiresart,self.hirescnv]
            else:
                self.files_to_delete+=[self.hiresart,self.hirescnv]


    def colorgals(self,master,color=None,colorsig=None,maxshift=None):
        """for use with the from_master method. Applies the color
                 effect only, not the scaling effect. """

        if ( (color is None or color == 0) and
            (maxshift is None or maxshift == 0) ):
            shutil.copyfile(master.artdata_file,self.artdata_file)
        else:
            
            #Open the input & output files
            f=open(master.artdata_file)
            out=open(self.artdata_file,'w')
            
            #Loop through & convert
            for line in f:
                x,y,mag,model,hlrad,axratio,posangle,flag=line.split()
                x=float(x)
                y=float(y)
                mag=float(mag)
                #color part
                if color:
                    thiscolor=random.gauss(color,colorsig)
                    mag+=thiscolor
                #shift part
                if maxshift:
                    x+=random.uniform(0,maxshift)
                    y+=random.uniform(0,maxshift)
                out.write("%f %f %7.3f %s %s %s %s %s\n"%
                          (x,y,mag,model,hlrad,axratio,posangle,flag))
            out.close()
            f.close()



    def from_master(self,master,tfitpars,verbose=0,srcnoise="no"):
        """ Generate an image with master pixel scale but target
        flux properties; then convolve it down. """
        #Don't do this here: do it in hte caller
        #Make the data file of the galaxies
        #self.colorgals(master,c['COLOR'],c['COLORSIG'])
        
        #Create the hires noiseless image
        iraf.artdata.dynrange=1.e5
        iraf.mkobjects(self.hiresart,
                       output="",
                       title="",
                       ncols=master.xdim,
                       nlines=master.ydim,
                       header=master.hdrfile,
                       background=0.0,
                       objects=self.artdata_file,
                       xoffset=0.0,
                       yoffset=0.0,
                       star="gaussian",
                       radius=0.1,
                       beta=2.5,
                       ar=1.0,
                       pa=0.0,
                       distance=1.0,
                       exptime=1.0,
                       magzero=self.magz,
                       gain=self.gain,
                       rdnoise=0.0,
                       poisson=srcnoise,
                       seed=2,
                       comments=1)

##         rsm.makenoiselessimage(self.artdata_file,
##                                os.path.splitext(self.hiresart)[0],
##                                self.magz,
##                                master.xdim,
##                                master.ydim,
##                                gain=self.gain,
##                                psffile=master.psffile,
##                                hdrfile=master.hdrfile,
##                                save=1) #always save
        
        #Convolve it with its psf
        iraf.fconvolve(self.hiresart,self.psffile,self.hirescnv)
        #Bin it down
        relscale=self.scale/master.scale
        iraf.blkavg(self.hirescnv,self.artimage,relscale,relscale,option='sum')
        
        #Add the result to the substrate
        iraf.imcalc(self.artimage+","+self.substrate,self.outimage,"im1+im2")


                                                  

    def movegals(self,master,color=None,colorsig=None):
        #Get the WCSs so we can convert
        masterim=fitsimage.Image(master.substrate)
        masterwcs=masterim.wcs

        imfile=fitsimage.Image(self.substrate)
        outwcs=imfile.wcs

        #Compute relative pixel scale
        relscale = outwcs.cd1_1/masterwcs.cd1_1
        
        #Open the input & output files
        f=open(master.artdata_file)
        out=open(self.artdata_file,'w')

        
        #Loop through & convert
        for line in f:
            x,y,mag,model,hlrad,axratio,posangle,flag=line.split()
            x=float(x)
            y=float(y)
            mag=float(mag)
            hlrad=float(hlrad)
            nx,ny=outwcs.xy2xy(x,y,masterwcs)
            nrad=hlrad/relscale
            if color is not None:
                thiscolor=random.gauss(color,colorsig)
                mag=mag+thiscolor
#                raise ValueError,'pause for debug'
            out.write("%10.2f %10.2f %7.3f %s %10.5f %s %s %s\n"%
                      (nx,ny,mag,model,nrad,axratio,posangle,flag))
        out.close()
        f.close()



class Masterimage(Photimage):
    def __init__(self,band,c,verbose=0):
        Photimage.__init__(self,band,c,verbose=verbose)
        self.galinfo_file=band+'_artdata_magcorr.dat'
        self.files_to_clean+=[self.galinfo_file,self.galinfo_file+'_2',
                              self.scat+'_hrc_trim.txt',
                              self.scat+'_lrf.txt']


    def gengals(self,artpar):
        rdpars=(artpar.rdist_sigma,
                artpar.rdist_r0/self.scale,
                artpar.rdist_mag0,
                artpar.rdist_beta)
        
        simtfit_util.makegals(artpar.ngal,
                self.galinfo_file,
                self.artdata_file,
                self.flagimage,
                artpar.maglow,
                artpar.maghigh,
                artpar.rmin/self.scale,
                artpar.rmax/self.scale,
                self.xdim,
                self.ydim,
                rdistfunc=artpar.rdistfunc,
                rdistpars=rdpars,
                edgebuffer=artpar.edgebuffer,
                circularflag=artpar.circularflag,
                diskfrac=artpar.diskfrac,
                flagmin=artpar.flagmin,
                magdist=artpar.magdist)

    def makeimage(self,artpars):
        """ Combined method """
        self.gen_substrate()
        self.gengals(artpars)
        self.addgals()

#--------------------------------------------------------
if __name__ == '__main__':
    print "Running sim_1psf\n"
    fname=sys.argv[1]
    main(argfile=fname)
