from pyraf import iraf
from iraf import artdata
from iraf import stsdas, toolbox, imgtools, analysis, fourier
import random
from math import *

qpar = {"devauc": (0,0.3, 0.9), "expdisk": (1,0.05, 0.01)}
devauc_correction = -0.13  

# Tests to see if galaxy is on the image
def test_flags(x,y,flagimage):
    """Tests to see if galaxy is (mostly) on the image"""
    f = flagimage+"[%d:%d,%d:%d]" % (x-5,x+5,y-5,y+5)
    s = iraf.imstat(f,fields="max",format=0,lower='INDEF',upper='INDEF',
	nclip=0,Stdout=1)
#   print f,s
    return int(float(s[0]))

# Creates a galaxy list file for iraf.artdata.mkobjects
def makegals(n,galfile,artfile,flagimage,maglow,maghigh,rmin,rmax,xmax,ymax,
       rdistfunc='uniform',rdistpars=[],
       edgebuffer=60,circularflag=0,diskfrac=0.5,flagmin=1,
       magdist=None):
    """Makes the galaxy list file; equal numbers of spirals & ellipticals"""
    i = 0
    galfile_forsextractor = "%s_2" % galfile
    f=open(galfile,"w")
    fa=open(artfile,"w")
    f2 = open(galfile_forsextractor,"w")

    if magdist is not None:
        mdist=random.sample(magdist,n)
        
    while i < n:
	type = ifgt(random.random(),diskfrac,"devauc","expdisk")

        if magdist is not None:
            mag=mdist[i]
        else:
            mag = maglow + random.random() * (maghigh-maglow)

        if rdistfunc == 'lognormal':
           re = rlognormal(rmin,rmax,mag,rdistpars)
        else:
           re = rmin + random.random() * (rmax-rmin)
	x = edgebuffer/2. + (xmax-edgebuffer)*random.random() 
	y = edgebuffer/2. + (ymax-edgebuffer)*random.random() 
	pa = 360. * random.random()
        if circularflag:
            ba = 1.0
        else:
            ba = get_axialratio(type)
        sini = random.random();
        inc = asin(sini);
        if test_flags(x,y,flagimage) <= flagmin:
	    fa.write("%10.2f %10.2f %8.3f %12s %8.3f %6.2f %6.2f no\n" 
		     % (x,y,mag,type,re,ba,pa))
            if type == "devauc":
	        f.write("%10.2f %10.2f %8.3f %12s %8.3f %6.2f %6.2f no\n" 
		     % (x,y,mag+devauc_correction,type,re,ba,pa))
                f2.write("%10.2f %10.2f %8.3f 1 %8.3f %6.2f %6.2f\n" 
		     % (x,y,mag,re,ba,pa))
            else:
	        f.write("%10.2f %10.2f %8.3f %12s %8.3f %6.2f %6.2f no\n" 
		     % (x,y,mag,type,re,ba,pa))
                f2.write("%10.2f %10.2f %8.3f 0 %8.3f %6.2f %6.2f\n" 
		     % (x,y,mag,re,ba,pa))

            i = i+1
    f.close()
    fa.close()
    f2.close()

#...................................................................



# Convolves with the PSF. 
def convolve(root,psffile,save=0):
    """convolve convolves infine with psf; for now just calls convolvebyquadrant
    """
    unconvolved = root+'_nopsf.fits'
    convolved = root+'.fits'
    os.rename(root+'.fits',unconvolved)  
    iraf.fconvolve(unconvolved,psffile,convolved)
    if not save:
      os.remove(unconvolved)

#........................................................................
# Gets a random axial ratio from a distribution of inclinations & true
# axial ratios
def get_axialratio(type):
    """Chooses a random axial ratio"""
    sini = random.random();
    qp = qpar[type]
    if qp[0] == 0:
        # Uniform axial ratio distribution
        q = (qp[2]-qp[1])*random.random()+qp[0]
    if qp[0] == 1:
        q = random.normalvariate(qp[1],qp[2])
        q = max(q,0.01) 
        q = min(q,1.)
    cosi = sqrt(1-sini**2)
    ba = sqrt(sini*sini + (q*cosi)**2)
    return ba

def ifgt(arg1,arg2,ans1,ans2):
    if arg1 > arg2:
        return(ans1)
    else:
        return(ans2)

#---------------------------------------------------------------
#Lognormal functions for radius distributions

def init_rlognormal(c):
  sigma = c['LOGNORMAL_SIGMA']
  r0 = c['LOGNORMAL_PEAK']/c['SCALE'][0]
  mag0 = c['LOGNORMAL_MAG0']
  beta = c['LOGNORMAL_BETA']
  return (sigma,r0,mag0,beta)

def rlognormal(rmin,rmax,mag,(sigma,r0,mag0,beta)):
  lumratio = 10**((mag-mag0)/-2.5)
  mu = log(r0)+beta*log(lumratio)
# print "beta, mag0, mag, lumratio, mu: %10.3f %6.2f %6.2f %10.4f %10.4f" % (beta, mag0, mag, lumratio, mu)  
  val = random.gauss(mu,sigma)
  radius = exp(val)
  return radius
