from pyraf import iraf
import os
import pyfits, numpy
from tfit import futil
# Script to use imcopy on a suitably shortened sextractor
# file in the format below to make cutouts around the objects.
#
# Uses imcopy to do the cutting out because that will transfer
# the header as well and write nice new ltv1/2 keywords. But
# then we have to subtract the sky and update the header, so we
# use pyfits to do that
# id  xc       yc     lx   ly  ux  uy        sky      flux (not used)
#714 660.969 1982.659 658 1980 664 1985 -0.000284976 0.103782
#
# Validated against IDL cutter-outer, 21 jan 2003.
def cut_them(catfile, imgfile, segfile, outdir,outcat=None,noclean=None,normalize=None,mycoldefs=None):
    assert os.path.isfile(catfile), "Catalog file does not exist: "+catfile
    assert os.path.isfile(imgfile), "Image file does not exist: "+imgfile
    assert os.path.isfile(segfile), "Image file does not exist: "+segfile

    if not outdir.endswith('/'): outdir+='/'
    if not os.path.isdir(outdir):
	os.mkdir(outdir)

    #Parse the catalog file header.
    colkeys=futil.catkeys(catfile)
    #Specify the columns we're going to want.
    
    if mycoldefs is not None:
        if type(mycoldefs) == type([]):
            mycols=mycoldefs
        else:
            colnames=open(mycoldefs).read()
            mycols=colnames.split()
            
    else:
        mycols=['NUMBER', 'X_IMAGE', 'Y_IMAGE', 
	    'XMIN_IMAGE', 'YMIN_IMAGE', 'XMAX_IMAGE','YMAX_IMAGE',
	    'BACKGROUND','FLUX_ISO']
    #Open the output file we're going to create for TFIT
    if outcat: 
	tfitcat=open(outcat,"w")
    #Loop through the file
    f=open(catfile,"r")
    counter=0
    for line in f:
	if not line.startswith('#'):
	    #process the line
	    counter+=1
	    id,xc,yc,lx,ly,ux,uy,sky,flux = futil.parseline(line,colkeys,mycols)
	    if outcat:
		print >>tfitcat, id,xc,yc,flux
	    #id,xc,yc,lx,ly,ux,uy,sky,flux = line.split()
	    #do the imcopy, of both image and segmap
	    subrange="["+lx+":"+ux+","+ly+":"+uy+"]"
	    ## subrange="["+`int(lx)+1`+":"+`int(ux)+1`+","+ \
## 		      `int(ly)+1`+":"+`int(uy)+1`+"]"
	    iraf.imcopy(imgfile+subrange,outdir+id+'_temp.fits')
	    iraf.imcopy(segfile+subrange,outdir+id+'_map.fits')

	    #Read the image & subtract the sky
	    nf=pyfits.open(outdir+id+'_temp.fits')
	    nf[0].data = nf[0].data - float(sky)
            
	    #Read the segmap & zero out the bad pixels
	    seg=pyfits.open(outdir+id+'_map.fits')[0].data
	    badpix = numpy.nonzero(seg != int(id))
	    nf[0].data[badpix] = 0.0

            #Normalize to one.
            totflux=nf[0].data.sum()
            if normalize:
                nf[0].data=nf[0].data/totflux
            
	    #Add the new keywords to the header
	    hdr=nf[0].header
	    hdr.update('totflux',totflux,comment='Total flux in object')
            if normalize:
                hdr.update('fluxnorm',pyfits.TRUE,comment='Normalized by totflux: new total = 1.0')
	    hdr.update('obskysub',float(sky),comment="Local sky value subtracted from data")
	    hdr.add_comment('Cutout from '+os.path.basename(imgfile),after='obskysub' )
	    hdr.add_comment('based on object catalog '+os.path.basename(catfile),after='obskysub' )
	    #Write out the resulting file & clean up the temporaries.
	    nf.writeto(outdir+id+'.fits')
	    if (noclean == None):
		#print noclean
		os.remove(outdir+id+'_temp.fits')
		os.remove(outdir+id+'_map.fits')

    f.close()
    if outcat:
	tfitcat.close()
    return "%d cutouts created."%counter
    

