import copy                #vanilla python
import numpy as N
import convolve as Convolve  #numarray extensions
from tfit import mynum               #numarray utilitites
from tfit import box,wcs,fitsimage   #general infrastructure
from tfit import tfit_sancheck       #tfit specific


True  = 1
False = 0

class MaxFlagError(Exception):
    """ Raise if any template pixels have a flag greater than or equal
    to the maximum flag value """
    pass

#----------------------------------------------------------------------
def getpadbox(hicut, rel_scale, hi_wcs, lo_wcs, int_border):
    """Pad the wcs_box to align with the coarse grid, and include the border.
    Must be passed wcs_box, not local_box"""
    #print "getpadbox4"
    #Don't work with the corner-puffed version.
    dx1 = (hicut.lx-1)%rel_scale
    dy1 = (hicut.ly-1)%rel_scale
    #Compute offset for upper corners
    dx2= (rel_scale - hicut.ux%rel_scale)%rel_scale
    dy2= (rel_scale - hicut.uy%rel_scale)%rel_scale
    #print "dx1,dy1,dx2,dy2 =" ,dx1,dy1,dx2,dy2 

    #Create a new box that stretches out to here
    hipad=box.Box(x1=hicut.lx-dx1,y1=hicut.ly-dy1,
		  x2=hicut.ux+dx2,y2=hicut.uy+dy2)

    #Compute a new border that will also align.
    newborder = pborder(int_border, rel_scale)


    #Put the border on.
    hnew=hipad.embed(newborder)

    #That's it.
    return hnew

#----------------------------------------------------------------------
def pborder(border,scale):
   """ Puff the border out to align with the coarse pixel grid """
   if border%scale:
      return border + (scale-border%scale)
   else:
      return border

#---------------------------------------------------------------------
def clip(range,num):
   """ Force num to occur within range. """ 
   bottom,top = range
   if bottom <= num <= top:
       return num, 0
   elif num < bottom:
       return bottom, 1
   elif num > top:
       return top, 1
#-----------------------------------------------------------------
def blur_one(hicut, lores, border, psf, scale_factor, fake=None,
             flag=None,maxflag=0):
    """ Create a template from a cutout """

 
    #Pad the box so it aligns with the coarse grid, and includes a border.
    if scale_factor != 1:
     #   print "scale != one: proceeding with getpadbox2"
	hnew = getpadbox(hicut.wcs_box, scale_factor, 
		      hicut.wcs, lores.wcs, border)
        tfit_sancheck.testbox(hnew,scale_factor)
    else:
      # print "Scale = 1: going straight to embed // & cornerpuff"
       hnew=hicut.wcs_box.embed(border)
    
    #...................................................................
    #Create a new wcs, and make sure it's okay
    newwcs = make_new_wcs(lores.wcs,hicut.wcs,hnew,scale_factor)
    tfit_sancheck.testwcs(newwcs)

    #...................................................................
    #Make a box that identifies the proper subset of the data
    newlocal=hicut.wcs.localpos(hnew)
    xdelta = hicut.wcs_box.lx - hnew.lx
    ydelta = hicut.wcs_box.ly - hnew.ly  #these must be positive
    newlocal.Move(xdelta,ydelta)
    nl2 = box.Box(newlocal.lx+xdelta, newlocal.ly+ydelta,
		  newlocal.lx+xdelta+hicut.wcs_box.Xdim()-1,
		  newlocal.ly+ydelta+hicut.wcs_box.Ydim()-1)
    #print "nl2 is: ",nl2

##     print "Original cutout crpix1/2: ",hicut.wcs.crpix1,hicut.wcs.crpix2
##     print "Modified newwcs crpix1/2: ",newwcs.crpix1, newwcs.crpix2

    #...................................................................
    #Now make a numarray object from that box
    work = N.zeros( (hnew.Ydim(),hnew.Xdim() ), N.float32)
    #Stick the actual data into the right place.
    work[nl2.Slice()] = hicut.d
    #...................................................................
    #Rearranging some steps so we can do a flag check here

    if scale_factor != 1:
        newimg=N.zeros( (hnew.Ydim()/scale_factor, hnew.Xdim()/scale_factor), N.float32)
    else:
        newimg=N.zeros( (hnew.Ydim(),hnew.Xdim()),N.float32)
    locut=assemble_template(hicut,newimg,newwcs)
                                   
    if flag is not None:
        #here's the test:
        subbox=flag.wcs.xy2xy_box(locut.local_box,locut.wcs).Intersection(flag.local_box)
        flagsub=flag.d[subbox.Slice()]
        if flagsub.max() >= maxflag:
            raise MaxFlagError
        locut.h.update('maxflag',flagsub.max(),
                               comment='/ Max flag value within template')

    #Now do the convolution
    if not fake:
        blur=Convolve.convolve2d(work,psf,fft=1) #Convolve it with the PSF
        if scale_factor != 1:
            #Bin it down
            newimg=mynum.rebin(blur,-scale_factor,conserve='true')  
        else:
            newimg=blur
        
    #Stick the data into the right place
    locut.d=newimg.astype(N.float32)
    return locut

    #...................................................................
##     # Do the actual convolution 
##     if fake is None:
## 	blur=Convolve.convolve2d(work,psf,fft=1) #Convolve it with the PSF
##     else:
## 	blur=work

##     #...................................................................
##     # Bin the convolved image down to the coarse pixel scale
##     if scale_factor != 1:
## 	newimg=mynum.rebin(blur,-scale_factor,conserve='true')  #Bin it down
##     else:
## 	newimg=blur
##     #...................................................................
##     #Now build a new fitsimage object out of the pieces.
##     locut = assemble_template(hicut, newimg, newwcs)
##     return locut

#----------------------------------------------------------------------
def assemble_template(hicut,newimg,newwcs):
    """ Return a fitsimage object of the new template"""

    locut = copy.copy(hicut)
#    locut.f = None  #No such file yet
    locut.filename = None  #Ditto filename
    locut.d = N.array(newimg,N.float32)       #Stick the data in

    #Define local box
    nx = newimg.shape[1] 
    ny = newimg.shape[0]
    locut.local_box = box.Box(x1=1,y1=1,x2=nx,y2=ny)  #Define local box

    #Define wcs, and insert values into header
    locut.wcs = newwcs     #already has the crpixes fixed above
    locut.wcs.insert(locut.h)  
    locut.h['naxis1'],locut.h['naxis2'] = \
			       locut.local_box.Xdim(), locut.local_box.Ydim()

    # Build the new wcs box, based on new wcs and new local_box
    locut.wcs_box = locut.wcs.wcspos(locut.local_box)  
    #
    # Return the new object
    return locut

#----------------------------------------------------------------------
def make_new_wcs(lores_wcs,hicut_wcs,hnew,scale):
    """Make a new wcs from the old one, the padded box, and the relative scale.
    Requires wcs_box, not local_box."""

    #copy the wcs
    newwcs = copy.deepcopy(lores_wcs)
    #compute the offset
    xdelta = (hnew.lx - 1)/scale
    ydelta = (hnew.ly - 1)/scale
    newwcs.set_crpix(-xdelta+0.5,-ydelta+0.5) 
    return newwcs
