import numpy,sys,os
from tfit import wcs,box,futil,fitsimage

class cellob:
    def __init__(self,line):
        """ Construct an object representing a cell.
        Includes magic column numbers.for line from cell file """
        # skip comment lines
        if line.startswith('#'):
            raise ValueError,"Line is a comment"

        #Make sure we have enough columns
        cols=line.split()
        if len(cols) != 14:
            msg="Not enough columns - skipping line %s"%line
            raise ValueError, msg

        #assign the fields
        self.id=float(cols[0])
        self.nobj=float(cols[1])

        lx=round(float(cols[4]))
        ly=round(float(cols[5]))
        ux=lx+round(float(cols[2]))
        uy=ly+round(float(cols[3]))
        self.box=box.Box(x1=lx,y1=ly,x2=ux,y2=uy)

        self.bg=float(cols[8])
        self.bgerr=float(cols[9])
        self.chi2=float(cols[10])
        self.dof=float(cols[11])
        self.reduced_chi2=float(cols[12])
        self.npix=float(cols[13])



def getcells(cellfile):
    """Make a dictionary of cells, indexed by cell id"""
    f=open(cellfile)
    d={}
    for line in f:
        if not line.startswith('#'):
            try:
                cell=cellob(line)
                d[cell.id]=cell
            except ValueError,e:
                pass
    f.close()
    return d

def bgmap(celldict,xdim,ydim):
    """ Make a bg map from the cellfile """
    
    #make the arrays to build.
    shape=(ydim,xdim)
    bgval=numpy.zeros(shape,numpy.float32)
    bgcount=numpy.zeros(shape,numpy.float32)

    #Loop through all cells
    for k in celldict:
        cell=celldict[k]
        bgval[cell.box.Slice()]+=cell.bg
        bgcount[cell.box.Slice()]+=1

    #Take the average value
    bgval=bgval/bgcount

    return bgval,bgcount

def main(cellfile,xdim,ydim):
    #Get the cell info
    celldict=getcells(cellfile)
    
    #Make the map
    bgval,bgcount=bgmap(celldict,xdim,ydim)

    #Save the results
    root,ext=os.path.splitext(cellfile)
    fname=root+'_cell_bg.fits'
    futil.writeimage(bgval,fname)

if __name__ == '__main__':
    cellfile,xdim,ydim=sys.argv.split()
    main(cellfile,float(xdim),float(ydim))
