#! /usr/bin/env python
"""
Cleaner packaging of some of the tfit stages.
For use with pipeline.py.

Changes from prior version:
  - assume booleans are real booleans in dictionary & can be
    tested against as truth values

"""
import os
import commands #Unix only!!
from tfit import tfit_diags, fitsimage, futil


#=====================================================
# First define the functions for each step.
#................................................................
def do_cutouts(hirescat,
               hiresfile,
               hiresseg,
               cutoutdir,
               cutoutcat,
               hirescatcols,
               normalize=True):
    import pyraf_cutout

    info=pyraf_cutout.cut_them(hirescat,
                                hiresfile,
                                hiresseg,
                                cutoutdir,
                                normalize=normalize,
                                mycoldefs=None)
    #This used to be optional. It's going to eventually
    #be entirely replaced by the use of sextutils. So just do it
    #all the time for now.
    
    futil.make_tfit_cat(hirescat,cutoutcat,
                      mycoldefs=None)
    return info
#................................................................
def do_convolve(d):
    """ Alternative: test whether d['psffile'] is a fits file (single)
    or not (multi) """
    
    if d['multikernels']:
        tlibrary.multikern(gridfile=d['lookupfile'],
                           indir=d['cutoutdir'],
                           outdir=d['templatedir'],
                           tfit_cat=d['cutoutcat'],
                           loresfile=d['loresfile'],
                           scale_factor=d['relscale'],
                           flagfile=d['flagfile'],
                           maxflag=d['maxflag'])
    else:
        tlibrary.process(d['cutout_dir'],
                         d['loresfile'],
                         d['psffile'],
                         d['relscale'],
                         d['templatedir'],
                         flagfile=d['flagfile'],
                         maxflag=d['maxflag'])
#..............................................................
def do_transform(d):
    """ needs to be split out from cull """
#..............................................................
def do_cull(d):
    tlibrary.cull(d['loresfile'],
                  d['hiresfile'],
                  d['templatecat'],
                  d['templatedir'])
#..............................................................
def excerpt_tfit_pars(d):
    out=open(d['tfitpars'],'w')
    out.write("LOW_RES_IMAGE        %s \n"%d['loresfile'])
    out.write("OBJECT_LIST          %s \n"%d['templatecat'])
    out.write("TEMPLATE_DIR         %s/ \n"%d['templatedir'])
    out.write("OUTPUT_LIST          %s \n"%d['tfitcat'])
    out.write("CELLFILE             %s \n"%d['tfitcell'])
    out.write("USE_ERRORS           %s \n"%d['errtype'].upper())
    out.write("ERR_MAP              %s \n"%d['loreserr'])
    out.write("RMS_CONSTANT         %f \n"%d['rmsconstant'])
    out.write("CELLMASK             %s \n"%d['cellmask'])
    out.write("FLOOR                %g \n"%d['maskfloor'])
    out.write("DITHERCELL           %s \n"%d['dithercell'])
    out.write("FLAG_MAP             %s \n"%d.get('loresflag','None'))
    out.write("MAX_FLAG             %d \n"%d['maxflag'])
    out.write("FIT_BACKGROUND       %s \n"%d['fitbackground'])
    out.write("BG_CONSTANT          %g \n"%d['bgconstant'])
    out.write("EXPORT_COVAR_INFO    %s \n"%d['writecovar'])
    out.write("COVARIANCE_FILE      %s \n"%d['tfitcovar'])
    out.write("XBUFFERSIZE          %d\n"%d['cell_xdim'])
    out.write("YBUFFERSIZE          %d\n"%d['cell_ydim'])
    out.write("FLAG_DISTANCE        %d\n"%d['cell_overlap'])
    out.close()
    return d['tfitpars']
#..............................................................
def do_fit(d):
    """ Regenerate the old-style parfile, and spawn to run the fit.
    Rely on the PATH being properly set to find the executable"""
    tfit_pars=excerpt_tfit_pars(d)
    tfit_verb = 'fitter'
    tfit_log = os.path.splitext(tfit_pars)[0]+'.log'
    tfit_cmd = ' '.join([tfit_verb,tfit_pars,'>',tfit_log])

    tfit_stat, tfit_msg = commands.getstatusoutput(tfit_cmd)
    if tfit_stat != 0:
        print tfit_msg
        raise ValueError, "TFIT failed: exit status %d"%os.WSTOPSIG(tfit_stat)
    return tfit_stat
#..............................................................
def do_diags(tfitcat,
             modelfile,
             loresfile,
             templatedir):

	sf=tfit_diags.getbest(tfitcat)
        collage_file=modelfile
	lores=fitsimage.Image(loresfile)
	products = tfit_diags.collage(lores.d.shape,templatedir,
                                      bigwcs=lores.wcs,mult=sf,
                                      getflags=True,finder=True)
        outcoll=products['collage']
	futil.writeimage(outcoll,collage_file,header=lores.h)
        resid=lores.d - outcoll
        residname='resid_'+os.path.basename(collage_file)
        if len(os.path.dirname(collage_file)) != 0:
            residname=os.path.join(os.path.dirname(collage_file),
                                   residname)
        futil.writeimage(resid,residname,header=lores.h)
        tfit_diags.addflags(tfitcat+'_best', products['flag'])
        futil.writeimage(products['id'],
                         residname.replace('resid','idmap'),
                         header=lores.h)
#..............................................................
def do_dance(d):
    tfit_dance.main(d['loresfile'],
                    d['modelfile'],
                    d['psffile'],
                    d['dzonesize'],
                    d['dmaxshift'],
                    d['psflookup'],
                    d['ddiagfile'],
                    d['dlogfile'])
