from tfit.pipeline import Stage
from tfit import tlibrary, tfit_dance, cull, tfit_control

#.......................................................
def StageFactory(StageName,d):
    """ return the right sort of Stage based on this name """
    StageName=StageName.lower()
    if StageName == 'cutout':
        
        args=(d['hirescat'],
              d['hiresfile'],
              d['hiresseg'],
              d['cutoutdir'],
              d['cutoutcat'],
              d['hicatcolnames'])
        kwargs={'normalize':d.get('normalize')}
               
        return Stage(StageName,
                     tfit_control.do_cutouts,
                     *args,
                     **kwargs)
                 
    if StageName == 'convolve':
        if d['multikernels']:
            if d['posframe'].startswith('lo'):
                catfile=d['lrfcat']
            else:
                catfile=d['cutoutcat']
            args=(d['psflookup'],
                  d['cutoutdir'],
                  d['templatedir'],
                  catfile,
                  d['loresfile'],
                  d['relscale'])
            kwargs={'flagfile':d.get('loresflag'), 
                    'maxflag':d.get('maxflag')}
 

            return Stage(StageName+'_multi',
                         tlibrary.multikern,
                         *args,
                         **kwargs)
                         
        else:
            args=(d['cutoutdir'],
                  d['loresfile'],
                  d['psffile'],
                  d['relscale'],
                  d['templatedir'])
            kwargs={'flagfile':d.get('loresflag'), 
                    'maxflag':d.get('maxflag'),
                    'savehi':d.get('save_hr'),
                    'hr_outdir':d.get('hr_templatedir')}

            return Stage(StageName,
                         tlibrary.process,
                         *args, **kwargs)
        
    if StageName=='transform':
        args=(d['loresfile'],
              d['hiresfile'],
              d['cutoutcat'],
              d['lrfcat'])
        return Stage(StageName,
                     cull.transform,
                     *args)
    
    if StageName=='cull':
        args= (d['loresfile'],
               d['lrfcat'],
               d['templatedir'],
               d['templatecat'])
        return Stage(StageName,
                     cull.cull,
                     *args)
    
    if StageName=='fit':
        return Stage(StageName,
                     tfit_control.do_fit,
                     d)
                     
    
    if StageName=='diags':
        args=(d['tfitcat'],
              d['modelfile'],
              d['loresfile'],
              d['templatedir'])

        return Stage(StageName,
                     tfit_control.do_diags,
                     *args)
    
    if StageName=='dance':
        args=(d['loresfile'],
              d['modelfile'],
              d['psffile'],
              d['dzonesize'],
              d['maxshift'],
              d['relscale'],
              d['psflookup'],
              d['ddiagfile'],
              d['dlogfile'])
        return Stage(StageName,
                     tfit_dance.main,
                     *args)

    if StageName=='plotdance':
        import plotdance   #import here to confine matplotlib
        args=(d['ddiagfile'],)
        print args
        return Stage(StageName,
                     plotdance.main,
                     *args)
    
    #If we get here, stage is undefined
    print "Sorry, %s stage has not yet been defined"%StageName
    return

