from distutils.core import setup, Extension
import sys, os.path, string, shutil, glob, os, re
from distutils.command.install_data import install_data



ver = sys.version_info
python_exec = 'python' + str(ver[0]) + '.' + str(ver[1])


def dolocal():
    """Adds a command line option --local=<install-dir> which is an abbreviation for
    'put all of thispackage in <install-dir>/thispackage'."""
    if "--help" in sys.argv:
        print >>sys.stderr
        print >>sys.stderr, " options:"
        print >>sys.stderr, "--local=<install-dir>    same as --install-lib=<install-dir>"
    for a in sys.argv:
        if a.startswith("--local="):
            dir =  os.path.abspath(a.split("=")[1])
            sys.argv.extend([
                "--install-lib="+dir,
                "--install-data="+dir,
                "--install-scripts="+os.path.join(dir,'bin')
                ])
            sys.argv.remove(a)

class smart_install_data(install_data):
    def run(self):
        install_cmd = self.get_finalized_command('install')
        self.install_dir = getattr(install_cmd, 'install_lib')
        return install_data.run(self)


pyfiles=glob.glob(os.path.join('lib','*.py'))
pymods=[]
for fname in pyfiles:
    tmp=os.path.basename(fname)
    pymods.append(os.path.splitext(tmp)[0])

docfiles=glob.glob(os.path.join('doc','*.txt'))
docfiles.extend(glob.glob(os.path.join('doc','*.html')))
        
def dosetup():
    r = setup(name = "tfit",
              version = "1.2.1",
              description  = 'Template-fitting Photometry',
              fullname     = 'AstroLib TFIT',
              license      = 'BSD',
              author = "Vicki Laidler",
              author_email = "help@stsci.edu",
              url = "http://projects.scipy.org/astropy/astrolib",
              platforms = ["Linux","Solaris","Mac OS X"],
              package_dir={'tfit':'lib'},
              packages=['tfit'],
              cmdclass = {'install_data':smart_install_data},
	      data_files = [('tfit',['LICENSE.txt']),
                            ('param',['lib/tfit_pipeline.param']),
                            ('doc',docfiles),
                            ('',['fitter'])],
              scripts = ['lib/tfit','lib/cell2region.py',
                         'fitter', 'lib/covar_proc.py',
                         'lib/set_covar_info.py',
                         'lib/applydance.py',
                         'lib/make_collage.py',
                         'lib/psf_collage.py']
              )

    return r

def main():
    args = set(sys.argv[1:])
    special = set(['build','build_scripts','install','install_scripts'])
    if args.intersection(special):
        #Build the fitter code with the makefile
        print "Building C++ code (fitter)"
        here=os.path.abspath(os.path.curdir)
        os.chdir('src')
        retstat=os.system('make >& %s'%os.path.join(here,'build_fitter.log'))
        os.chdir(here)
        if retstat != 0:
            raise OSError('fitter make failed: see build_fitter.log for more details.')
        
        #Link tfit_pipeline.py to plain old "tfit" for ease of use
        os.chdir('lib')
        try:
            os.symlink('tfit_pipeline.py','tfit')
        except OSError,e:
            if e.strerror != 'File exists':
                print "Error creating symlink tfit_pipeline.py -> tfit:"
                print str(e)
                print "Proceeding"
        os.chdir(here)

    dolocal()
    dosetup()


if __name__ == "__main__":
    main()

