#include <iostream.h> 
#include <iomanip.h>
#include <math.h>
#include <fstream.h>
#include <string.h>
#include <stdlib.h>
#include <assert.h>
#include <time.h>
#include "fitsio.h"
#include "nrutil.h"
#include "define.h"
#include "box.h"
#include "fits.h"
#include "fitsmanip.h"


Fits::Fits() {
    title = "UNNAMED";
}
  
Fits::Fits(int a, int b) {
    title = "Temporary";
    PixType = TFLOAT;
    xmin=0; ymin=0; totflux=0.0;
    xsize = a; ysize = b;
    //   xcorner = 0; ycorner = 0.0;
    xmax=xsize+xmin-1; ymax=ysize+ymin-1;
    xyrange.Set(xmin, ymin, xmax, ymax);

    data = matrix(ymin,ymax,xmin,xmax);
    for (int i = xmin; i <= xmax; i++) 
      for (int j = ymin; j <= ymax; j++)
	data[j][i] = 0.0;
}

Fits::Fits(int x0, int x1, int y0, int y1) {
    title = "Temporary";
    PixType = TFLOAT;
    xmin = x0; ymin = y0; totflux=0.0;
    // xcorner = 0; ycorner = 0.0;
    xsize = x1-x0+1; ysize = y1-y0+1;
    xyrange.Set(xmin, ymin, xmax, ymax);
    xmax=x1; ymax=y1;
    data = matrix(y0,y1,x0,x1);
    for (int i = xmin; i <= xmax; i++) 
      for (int j = ymin; j <= ymax; j++)
	data[j][i] = 0.0;
}

Fits::Fits(int x0, int x1, int y0, int y1,int pixtype) {
  int i,j;
  title = "Temporary";
  PixType = pixtype; totflux=0.0;
  xmin = x0; ymin = y0;
  xsize = x1-x0+1; ysize = y1-y0+1;
  // xcorner = 0; ycorner = 0.0;
  xmax=x1; ymax=y1;
  xyrange.Set(xmin, ymin, xmax, ymax);
  switch(PixType) {
  case TFLOAT :
    data = matrix(y0,y1,x0,x1);
    for (i = xmin; i <= xmax; i++) 
      for (j = ymin; j <= ymax; j++)
	data[j][i] = 0.0;
    break;
  case TINT :
    idata = imatrix(ymin,ymax,xmin,xmax);
    for (i = xmin; i <= xmax; i++) 
      for (j = ymin; j <= ymax; j++)
	idata[j][i] = 0;
    break;
  case TLONG :
    ldata = lmatrix(ymin,ymax,xmin,xmax);
    for (i = xmin; i <= xmax; i++) 
      for (j = ymin; j <= ymax; j++)
	ldata[j][i] = 0;
    break;
  default:
    cout << "\tUNKNOWN PIXTYPE = " << pixtype << " IN FITS " << endl;
  }
} 

Fits::Fits(int x0, int x1, int y0, int y1,int pixtype, float initval) {
  int i,j;
  title = "Temporary";
  PixType = pixtype; totflux=0.0;
  xmin = x0; ymin = y0;
  xsize = x1-x0+1; ysize = y1-y0+1;
  // xcorner = 0; ycorner = 0.0;
  xmax=x1; ymax=y1;
  xyrange.Set(xmin, ymin, xmax, ymax);
  switch(PixType) {
  case TFLOAT :
    data = matrix(y0,y1,x0,x1);
    for (i = xmin; i <= xmax; i++) 
      for (j = ymin; j <= ymax; j++)
	data[j][i] = initval;
    break;
  case TINT :
    idata = imatrix(ymin,ymax,xmin,xmax);
    for (i = xmin; i <= xmax; i++) 
      for (j = ymin; j <= ymax; j++)
	idata[j][i] = 0;
    break;
  case TLONG :
    ldata = lmatrix(ymin,ymax,xmin,xmax);
    for (i = xmin; i <= xmax; i++) 
      for (j = ymin; j <= ymax; j++)
	ldata[j][i] = 0;
    break;
  default:
    cout << "\tUNKNOWN PIXTYPE = " << pixtype << " IN FITS " << endl;
  }
} 



Fits::Fits(int a, int b, char *t, int pixtype) {
  int i,j;
  title = t;
  PixType = pixtype; totflux=0.0;
  xmin=0; ymin=0;
  xsize = a; ysize = b;
  xmax=xsize+xmin-1; ymax=ysize+ymin-1;
  //xcorner = 0; ycorner = 0.0;
  xyrange.Set(xmin, ymin, xmax, ymax);
  switch (PixType) {
  case TFLOAT :
    data = matrix(ymin,ymax,xmin,xmax);
    for (i = xmin; i <= xmax; i++) 
      for (j = ymin; j <= ymax; j++)
	data[j][i] = 0;
    break;
  case TINT :
    idata = imatrix(ymin,ymax,xmin,xmax);
    for (i = xmin; i <= xmax; i++) 
      for (j = ymin; j <= ymax; j++)
	idata[j][i] = 0;
    break;
  case TLONG :
    ldata = lmatrix(ymin,ymax,xmin,xmax);
    for (i = xmin; i <= xmax; i++) 
      for (j = ymin; j <= ymax; j++)
	ldata[j][i] = 0;
    break;
  default:
    cout << "\tUNKNOWN PIXTYPE = " << pixtype << " IN FITS " << t << endl;
  }
  if (PixType == TFLOAT) 
    cout << "\tFits: " << title << " -- Pixel Type: FLOAT (32 bit)" << endl;
  if (PixType == TINT) 
    cout << "\tFits: " << title << " -- Pixel Type: INT (16 bit)" << endl;
  if (PixType == TLONG) 
    cout << "\tFits: " << title << " -- Pixel Type: INT (32 bit)" << endl;
}

Fits::~Fits() {
  switch(PixType) {
  case TFLOAT :
    free_matrix(data,ymin,ymax,xmin,xmax);
    break;
  case TINT :
    free_imatrix(idata,ymin,ymax,xmin,xmax);
    break;
  case TLONG :
    free_lmatrix(ldata,ymin,ymax,xmin,xmax);
    break;
  default:
    cout << "Something weird in Fits " << title << endl;
    cout << "pixtype = " << PixType << endl;
  }
}

char* Fits::Title() {return title;}

void Fits::newTitle(char *t) { title = t; } 

int Fits::rnd(float x) {
  float h = ceil(x);
  return h - x > 0.5 ? (int)x : (int)h; 
}

// bool Fits::isInside(float XMIN, float YMIN, float XMAX, float YMAX) {
//   return (xcorner >= XMIN && (xcorner + xsize) <= XMAX &&
//           ycorner >= YMIN && (ycorner + ysize) <= YMAX );
// }

void Fits::scale(float C1, float C2, Fits *F) {
  int i,j;
  switch(PixType) {
  case TFLOAT :
    for (i = xmin; i <= xmax; i++) 
      for (j = ymin; j <= ymax; j++) 
	data[j][i] = C1 + C2 * F->data[j][i];
    break;
  case TINT :
    for (i=xmin; i <= xmax; i++) 
      for (j = ymin; j <= ymax; j++) 
	idata[j][i] = rnd(C1 + C2 * F->idata[j][i]);
    break;
  case TLONG :
    for (i=xmin; i <= xmax; i++) 
      for (j = ymin; j <= ymax; j++) 
	ldata[j][i] = rnd(C1 + C2 * F->ldata[j][i]);
    break;
  }
}

void Fits::resetPix(int fromVal, int toVal) { // Used only in funcs_group
  int i,j;
  switch(PixType) {
  case TFLOAT :
    for (i=xmin;i<=xmax;i++)
      for (j=ymin;j<=ymax;j++) 
	if (data[j][i] == (float)fromVal) data[j][i] = (float)toVal;
    break;
  case TINT :
    for (i=xmin;i<=xmax;i++)
      for (j=ymin;j<=ymax;j++) if (idata[j][i] == fromVal) idata[j][i] = toVal;
    break;
  case TLONG :
    for (i=xmin;i<=xmax;i++)
      for (j=ymin;j<=ymax;j++) if (ldata[j][i] == fromVal) ldata[j][i] = toVal;
    break;
  }
}

void Fits::convolve(Fits *F, Fits *conv) {
  float filter,val,norm,rval=0.0;
  unsigned int ifilter,inorm,ival;
  int i,j,k,l,BOUNDARY_VALUE;

  // **********************************************************
  // PIXTYPE == TFLOAT : Normal convolution will be performed using the
  //   convfile kernel.
  // PIXTYPE == TINT : Each object (in the segmentation map) will be grown 
  //   using the convfile kernel.
  // **********************************************************
  // BOUNDARY_VALUE == 
  // to calculate the value off the boundary - 
  //   0 : use zero
  //   1 : set to last border pixel
  //   2 : reflection
  //   !!!!!!!!!! Currently only 0 is supported !!!!!!!!!!!!
  // **********************************************************
  // New Patch added 24 Jan 1999:  If all elements of a convolution
  // element are zero than just make it zero -- don't waste time.
  // 
  // Patch added 4 April 2000
  // This file now handles convolution kernels in FITS format (the file
  // must end in .fits to be read as a fits image)
  // 
  // **********************************************************

  switch(F->PixType) {
  case TFLOAT :
    for (i = F->xmin; i <=F->xmax; i++) 
      for (j = F->ymin; j <=F->ymax; j++) {
	filter = 0.0;
	norm = 0.0;
	for (k = conv->xmin; k<=conv->xmax; k++) 
	  for (l = conv->ymin; l<=conv->ymax; l++) {
	    if (((j+l) > F->ymax) || ((i+k) > F->xmax) || 
		((j+l) < F->ymin) || ((i+k) < F->xmin)) {
	      val = BOUNDARY_VALUE == 0 ? 0.0 : 0.0;
	    } else  {
	      val = F->data[j+l][i+k];
	      filter += conv->data[l][k] * val;
	      norm += conv->data[l][k];
	    }
	  }
	this->data[j][i] = filter/norm;
      }
    break;
  case TINT :
    srand(time(NULL));
    for (i = F->xmin; i <= F->xmax; i++) 
      for (j = F->ymin; j <= F->ymax; j++) {
	ifilter = 0;
	ival = 0;
	for (k = conv->xmin; k<=conv->xmax; k++) 
	  for (l = conv->ymin; l<=conv->ymax; l++) {
	    if (((j+l) > F->ymax) || ((i+k) > F->xmax) ||
		((j+l) < F->ymin) || ((i+k) < F->xmin)) {
	      ival = BOUNDARY_VALUE == 0 ? 0 : 0;
	    } else if (conv->idata[l][k] > 0) {
	      ival = F->idata[j+l][i+k];
	      if ((ival > 0) && (ifilter > 0) && (ifilter != ival)) {
		rval = (float)(rand()/(float)RAND_MAX);
		ifilter = rval > 0.5 ? ifilter : ival;
	      } else if (ifilter==0) ifilter = ival;
	    }
	  }
	idata[j][i] = ifilter;
      }
    break;
  default :
    cout << "% WARNING -- SOMETHING WEIRD IN CONVOLVE -- PIXTYPE = ";
    cout << PixType << endl;
  }
}

ostream &operator<<(ostream &stream, Fits &fits) {
  int i,j;
  stream << fits.title << endl;
  switch (fits.PixType) {
  case TFLOAT :
    for (i = fits.xmin; i <=fits.xmax; i++) {
      for (j = fits.ymin; j <=fits.ymax; j++)
	stream << fits.data[j][i] << " ";
      stream << endl;
    }
    break;
  case TINT :
    for (i = fits.xmin; i <=fits.xmax; i++) {
      for (j = fits.ymin; j <=fits.ymax; j++)
	stream << fits.idata[j][i] << " ";
      stream << endl;
    }
    break;
  case TLONG :
    for (i = fits.xmin; i <=fits.xmax; i++) {
      for (j = fits.ymin; j <=fits.ymax; j++)
	stream << fits.ldata[j][i] << " ";
      stream << endl;
    }
    break;
  }
  return stream;
}


void Fits::subtract(float bg) {
  int i,j;
  switch(PixType) {
  case TFLOAT :
    for (i = xmin; i<=xmax; i++)
      for (j = ymin; j<=ymax; j++) {
	data[j][i] = data[j][i] - bg;
      }
    break;
  case TINT :
    for (i = xmin; i<=xmax; i++)
      for (j = ymin; j<=ymax; j++) {
	data[j][i] = idata[j][i] - (int)bg;
      }
    break;
  case TLONG :
    for (i = xmin; i<=xmax; i++)
      for (j = ymin; j<=ymax; j++) {
	data[j][i] = ldata[j][i] - (long)bg;
      }
    break;
  default :
    cout << "In Fits.subtract(float) -- (Fits) pixtype = " << PixType << endl;
  }
}

  // Note that both kinds of boxes are indexed from zero.
  // The long vector is indexed from 1.
  // This is a dangerous assumption that should be handled differently
  // but I'll do it this way for now.

// Long Integer version for dealing with flag map.
void Fits::Flatten(Box frame, long* holder) {
  int i,j,k,pcounter;
  // There are 3 kinds of flattening.
  // In case 1, the frame box is identical to the FITS box. This 
  // is the case of a cell that has been grown to precisely contain a cutout.
  // In case 2, the frame box is bigger than the FITS box. This
  // is the case of a cell being extracted from the image.
  // In case 3, the Fits box is bigger than the frame box. This
  // is the case of a template cutout inside a range.
  pcounter=0;

  if (frame == xyrange) { // case 1: frame is same size as image
    for (i=xmin; i<=xmax; i++) 
      for (j=ymin; j<=ymax; j++) {
	k=i*frame.Ydim() + j + 1;
	pcounter++;
	holder[k]=ldata[j][i];
      }
  } else if (frame.IsInside(xyrange) ) { // case 2: frame inside image
//     cout << "Case 1: frame inside image. " << endl;
//     cout << "xrange = " << frame.lx << "<=" << frame.ux << endl;
//     cout << "yrange = " << frame.ly << "<=" << frame.uy << endl;

    for (i=frame.lx; i<=frame.ux; i++)
      for (j=frame.ly; j<=frame.uy; j++) { //just added 1
        k = (i-frame.lx)*frame.Ydim() + (j-frame.ly) + 1 ;
        pcounter++;  
        holder[k] = ldata[j][i];
      }
  } else if (xyrange.IsInside(frame) ) { //case 3: image inside frame
//     cout << "Case 2: image inside frame. " << endl;
//     cout << "xrange = " << xmin+frame.lx << "<=" << xmax+frame.lx << endl;
//     cout << "yrange = " << ymin+frame.ly << "<=" << ymax +frame.ly << endl;

    for (i=xmin; i<=xmax; i++) 
      for (j=ymin; j<=ymax; j++) {
        //cout << "i, j, k: " << i<< j<< k << endl;
        k = (i+frame.lx)*frame.Ydim() + (j+frame.ly) + 1;
        pcounter++;
        holder[k] = ldata[j][i];
      }
  } else {
    cout << "Problem, alien geometry with " << title << endl;
  }
  // Debugging section
  pcounter=0;

}

void Fits::Flatten(Box frame, double* holder) {
  int i,j,k,pcounter;
  // There are 3 kinds of flattening.
  // In case 1, the frame box is identical to the FITS box. This 
  // is the case of a cell that has been grown to precisely contain a cutout.
  // In case 2, the frame box is bigger than the FITS box. This
  // is the case of a cell being extracted from the image.
  // In case 3, the Fits box is bigger than the frame box. This
  // is the case of a template cutout inside a range.
  pcounter=0;

  if (frame == xyrange) { // case 1: frame is same size as image
    for (i=xmin; i<=xmax; i++) 
      for (j=ymin; j<=ymax; j++) {
	k=i*frame.Ydim() + j + 1;
	pcounter++;
	holder[k]=data[j][i];
      }
  } else if (frame.IsInside(xyrange) ) { // case 2: frame inside image
//     cout << "Case 1: frame inside image. " << endl;
//     cout << "xrange = " << frame.lx << "<=" << frame.ux << endl;
//     cout << "yrange = " << frame.ly << "<=" << frame.uy << endl;

    for (i=frame.lx; i<=frame.ux; i++)
      for (j=frame.ly; j<=frame.uy; j++) { //just added 1
        k = (i-frame.lx)*frame.Ydim() + (j-frame.ly) + 1 ;
        pcounter++;  
        holder[k] = data[j][i];
      }
  } else if (xyrange.IsInside(frame) ) { //case 3: image inside frame
//     cout << "Case 2: image inside frame. " << endl;
//     cout << "xrange = " << xmin+frame.lx << "<=" << xmax+frame.lx << endl;
//     cout << "yrange = " << ymin+frame.ly << "<=" << ymax +frame.ly << endl;

    for (i=xmin; i<=xmax; i++) 
      for (j=ymin; j<=ymax; j++) {
        //cout << "i, j, k: " << i<< j<< k << endl;
        k = (i+frame.lx)*frame.Ydim() + (j+frame.ly) + 1;
        pcounter++;
        holder[k] = data[j][i];
      }
  } else {
    cout << "Problem, alien geometry with " << title << endl;
  }
  // Debugging section
  pcounter=0;

}


void Fits::Flatten(Box frame, double** holder, long row) {
  // Alas, we need this version too because the array is
  // being stored in Fortran-like array order.
  int i,j,k, ii, jj, pcounter;
  // There are 3 kinds of flattening.
  // In case 1, the frame box is identical to the FITS box. This 
  // is the case of a cell that has been grown to precisely contain a cutout.
  // In case 2, the frame box is bigger than the FITS box. This
  // is the case of a cell being extracted from the image.
  // In case 3, the Fits box is bigger than the frame box. This
  // is the case of a template cutout inside a range.

  pcounter=0;

  if (frame == xyrange) { // case 1: frame is same size as image
    for (i=xmin; i<=xmax; i++) 
      for (j=ymin; j<=ymax; j++) {
	k=i*frame.Ydim() + j + 1;
	pcounter++;
	holder[k][row]=data[j][i];
      }
  } else
  if (frame.IsInside(xyrange) ) { // case 2: frame inside image
//     cout << "Case 1: frame inside image. " << endl;
//     cout << "xrange = " << frame.lx << "<=" << frame.ux << endl;
//     cout << "yrange = "<< frame.ly << "<=" << frame.uy << endl;
    for (i=frame.lx; i<=frame.ux; i++)
      for (j=frame.ly; j<=frame.uy; j++) {
	//        k = (i-frame.lx)*(j-frame.ly) + (j-frame.ly) ;
        k = (i-frame.lx)*frame.Ydim() + (j-frame.ly) + 1;
        //cout << "i, j, k: " << i<< j<< k << endl;    
        pcounter++;  
        holder[k][row] = data[j][i];
      }
  } else if (xyrange.IsInside(frame) ) { //case 3: image inside frame
//     cout << "Case 2: image inside frame. " << endl;
//     cout << "xrange = " << xmin-frame.lx+xyrange.lx << "<=" << xmax-frame.lx+xyrange.lx << endl;
//     cout << "yrange = " << ymin-frame.ly+xyrange.ly << "<=" << ymax -frame.ly+xyrange.ly << endl;

    for (i=xmin; i<=xmax; i++) 
      for (j=ymin; j<=ymax; j++) {
        ii = i + xyrange.lx - frame.lx;
        jj = j + xyrange.ly - frame.ly;
        k = (ii)*frame.Ydim() + (jj) + 1;
        pcounter++;
        holder[k][row] = data[j][i];
      }
  } else {
    cout << "Problem, alien geometry with " << title << endl;
  }
  // Debugging section
  pcounter=0;


}
