#include <iostream.h>
#include <iomanip.h>
#include <math.h>
#include <fstream.h>
#include <string.h>
#include <stdlib.h>
#include <assert.h>
#include "fitsio.h"
#include "nrutil.h"
#include "define.h"
#include "box.h"
#include "fits.h"
#include "fitsmanip.h"

Fits *FitsManip::ReadFits(fitsfile *fimage,char ImageFile[],int *status,
                                           char title[], int sub) {
  long firstPixel=FIRSTPIXEL, nulVal=NULVAL;
  int anyVal=ANYVAL;
  int i,j,XSIZE=0, YSIZE=0, BITPIX=0, PIXTYPE=0, xoffset, yoffset;
  float CRPIX1=0.0, CRPIX2=0.0, TOTFLUX=0.0;
  char *xaxis,*yaxis,*pixtype,*xcorner,*ycorner,*totflux,comments[COMMENT_SIZE];
  xaxis = "NAXIS1"; yaxis = "NAXIS2"; pixtype = "BITPIX";
  xcorner = "CRPIX1"; ycorner="CRPIX2"; totflux="TOTFLUX";
  Fits *Image;
  int *value, sumflux=0;

  *status=0;
  try {
    fits_open_file(&fimage, ImageFile, READONLY, status);
    if (*status != 0) throw status;
    fits_read_key(fimage,TINT,xaxis,&XSIZE,comments,status);
    if (*status != 0) throw status;
    fits_read_key(fimage,TINT,yaxis,&YSIZE,comments,status);
    if (*status != 0) throw status;
    fits_read_key(fimage,TFLOAT,xcorner,&CRPIX1,comments,status);
    if (*status != 0) throw status;
    fits_read_key(fimage,TFLOAT,ycorner,&CRPIX2,comments,status);
    if (*status != 0) throw status;
    fits_read_key(fimage,TINT,pixtype,&BITPIX,comments,status);
    if (*status != 0) throw status;
    fits_read_key(fimage,TFLOAT,totflux,&TOTFLUX,comments,status);
    if (*status == KEY_NO_EXIST) {  // then we need to add it up by hand
      sumflux=KEY_NO_EXIST;  
      *status = 0;
    }


  }
  catch (int *value) {
    fits_report_error(stderr,*status);
    cout << "Program aborting..." << endl;
    exit(3);
  }
  setPixType(&PIXTYPE,BITPIX);
  Image = new Fits(XSIZE,YSIZE,title,PIXTYPE);
  if (!Image) {
    cout << "Unable to allocate memory for " << title << " data, Sorry :(\n";
    exit(2);
  }
  Image->crpix1 = CRPIX1;
  Image->crpix2 = CRPIX2;
  //Image->xcorner=CRPIX1;
  //Image->ycorner=CRPIX2;
  //***********************************************************************************
  if (sub){
//     xoffset=Image->rnd(CRPIX1)-1; //-1; 
//     yoffset=Image->rnd(CRPIX2)-1; //-1;

//     xoffset = int(CRPIX1);
//     yoffset = int(CRPIX2);

//     xoffset = int(CRPIX1 < 0 ? floor(CRPIX1) : ceil(CRPIX1) );
//     yoffset = int(CRPIX2 < 0 ? floor(CRPIX2) : ceil(CRPIX2) );

    //    Image->xyrange.Set(0,0,XSIZE-1,YSIZE-1); //Force C++ notation for pixel indices at this point

    //    Image->xyrange.Move(-xoffset,-yoffset);


    Image->xyrange.Set(CRPIX1,CRPIX2,CRPIX1+XSIZE-1,CRPIX2+YSIZE-1);
    // This will give half-pixel ranges, but the library routine will fix it.
  }
  //  Image->xyrange.Set(CRPIX1, CRPIX2, CRPIX1+XSIZE-1, CRPIX2+YSIZE-1);
  // Watch it. This is going to give you half-pixel values. 
  // Probably want some kind of "poof out to the corners" functionality here too.
  // Or, er, rather, no, the inverse. CRPIX1 = 32.5 => pixel 1 index is 33 in
  // FITS notation, 32 in c++ notation. 
  //***********************************************************************************
  switch(PIXTYPE) {
  case TFLOAT :
    fits_read_img(fimage,PIXTYPE,firstPixel,(long)(Image->xsize*Image->ysize),
		  &nulVal,Image->data[0],&anyVal,status);
    if (sumflux != 0) {
      for (i=Image->xmin;i<=Image->xmax;i++) 
        for (j=Image->ymin; j<=Image->ymax; j++) { 
          TOTFLUX = TOTFLUX+Image->data[j][i];
      }      
    }
    Image->totflux=TOTFLUX;

    break;
  case TINT :
    fits_read_img(fimage,PIXTYPE,firstPixel,(long)(Image->xsize*Image->ysize),
		  &nulVal,Image->idata[0],&anyVal,status);
    if (sumflux != 0) {
      for (i=Image->xmin;i<=Image->xmax;i++) 
        for (j=Image->ymin; j<=Image->ymax; j++) { 
          TOTFLUX = TOTFLUX+(float)Image->idata[j][i];
      }
      Image->totflux=TOTFLUX;
    }
  case TLONG :
    fits_read_img(fimage,PIXTYPE,firstPixel,(long)(Image->xsize*Image->ysize),
		  &nulVal,Image->ldata[0],&anyVal,status);
    if (sumflux != 0) {
      for (i=Image->xmin;i<=Image->xmax;i++) 
        for (j=Image->ymin; j<=Image->ymax; j++) { 
          TOTFLUX = TOTFLUX+(float)Image->ldata[j][i];
      }
      Image->totflux=TOTFLUX;
    }

    break;
  default:
    cout << "Unknown PIXTYPE: " << PIXTYPE << " in READFITS" << endl;
  }

  Image->xyrange.Set(Image->xmin, Image->ymin, Image->xmax, Image->ymax);
  fits_close_file(fimage,status);
  fits_report_error(stderr,*status);
  return Image;
}


//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

int FitsManip::CheckNorm(fitsfile *fimage, char ImageFile[], int *status) {
  char *fluxnorm; int ans, retval;
  char comments[COMMENT_SIZE];
  fluxnorm = "FLUXNORM";

  *status=0;
  try {
    fits_open_file(&fimage, ImageFile, READONLY, status);
    if (*status != 0) throw status;
    fits_read_key(fimage,TLOGICAL,fluxnorm,&ans,comments,status);
    if (*status == KEY_NO_EXIST) { // then it's not normalized
      retval = FALSE; 
    } else {
      retval = ans;
    }
  }
  catch (int *value) {
    fits_report_error(stderr,*status);
    cout << "Program aborting..." << endl;
    exit(3);
  }

    fits_close_file(fimage,status);
    fits_report_error(stderr,*status);
    return retval;
}
