#include <iostream>
#include <iomanip.h>
#include <math.h>
#include <fstream.h>
#include <string.h>
#include <stdlib.h>
#include <assert.h>
#include "fitsio.h"
#include "nrutil.h"
#include "define.h"
#include "fits.h"
#include "structs.h"
#include "par.h"
#include "fitsmanip.h"
#include "funcs.h"
#include "tlibrary.h"
#include <vector>
using std::vector;

double Funcs::getmaxcomp( double **V, int ndim, int tcount ) 
{
  int j;
  double maxcomp;

  maxcomp = fabs(V[tcount][tcount]);
  for (j=1; j<=ndim; j++){
    if (fabs(V[j][tcount]) > maxcomp) {maxcomp=V[j][tcount];}
  }

  return maxcomp;
}

int * Funcs::cellmask( float mask_floor, int max_flag, long *flag, double **temp, int crowdsize, int npix, int &new_npix)
{
  int i,j;
  float sum;
  int *mask;

  // Make a new output array
  mask = ivector(1,npix);
  // Walk through the templates
  new_npix = 0;
  for (j=1; j<=npix; j++) {
    // Apply the flag logic first
    if (flag[j] <= max_flag) {
      // If the flag image shows good data, examine the templates
      sum=0.0;
      for (i=1; i<=crowdsize; i++) { 
	if (temp[j][i] > mask_floor){
	  sum+=temp[j][i];}
      }

      if (sum != 0.0) 
	{mask[j]=1;
	new_npix++;
	}  
      else
	{mask[j]=0;}
    } 
  else {mask[j]=0;} // Otherwise, just mask it out.
  }
    return mask;
}

void Funcs::walkGrid(Fits *I, Fits *E, Fits *F, tLibrary *library, int USESIG,
		    char outfile[], char cellfile[], Grid *BigGrid, 
                    float RMS_CONSTANT, 
		     int use_cellmask, float mask_floor, int dithercell,
		     int max_flag, int fit_to_bg,
		    int exportCovariance, char CovarFile[], 
		    int XBUFFERSIZE, int YBUFFERSIZE,
		    int FLAGDIST, int *status) {

  /*********************************************************************
   *
   *  The Fits images are designated as:
   *    I: The full LoRes image (x,y) values over the whole image
   *********no longer needed?    T: The full template (Hi-Res image)
   *    E: The full weight map on the lores image
   *    F: The full flag map on the lores image
   *********  SegMap:  The full segmentation map from SExtractor (or similar)
   *    Map:  An image containing 1's where all pixels belonging to 
   *          group N and 0's everywhere else.  It's size is from SetBound
   *    Mask:  The background subtracted image from T with reduced bounds
   *    Image:  The reduced bounds of the image to be fit.
   *    ObjMap:  a temporary fits to hold the pixels belonging just to 
   *          one object -- used to determine which objects influence
   *          which pixels
   *********    ConObjMap: the convolve version of ObjMap
   *
   *********************************************************************/

  register int a,b,i,j,k,l,m,n;
  static long GRIDCOUNT = 0;
  static int XX=1, YY=2;
  long 
    crowdsize, growcount, *imflag,
    XMIN, XMAX,
    YMIN, YMAX,
    NXGRIDS,NYGRIDS;
  int 
    flag=0,
    flaggedCount=0,
    groupedCount=0,
    myi, myj, dither, xditherwidth, yditherwidth,
    x0, y0, 
    xlo, xhi, ylo, yhi, cx, cy,
    bxlo=0, bxhi=0, bylo=0, byhi=0,
    xcorner, ycorner,
    *lowcell, *highcell,
    npix, cellindex, pcounter,
    *dofflag, *mask, new_npix=0;
  char pme[STRING_LENGTH];
  float 
    xx, yy, obx, oby, 
    xc, yc, 
    xpos=0.0, ypos=0.0, 
    background=0.0, 
    fwhm=0.0,
    flux=0.0,
    isoflux,
    XCEN, YCEN, rcell;
  double 
    **covar, **svd_U, **svd_V, **temp, **newtemp,
    chi2=0.0, totchi2=0.0, totflux=0.0, oberr, maxcomp,
    *svd_W, 
    *image,*newimage, 
    *err, *newerr;
  ofstream datout, covout, cellout;
  Fits *thisone;
  Par *dof; 
  List *theList;
  gNode *step, *pnt, *item;
  FitsManip fitsmanip;
  
  fitsfile fthis;

  Box cell, newcell;
  vector <gNode *>  crowd;
  //  vector<long>::iterator tcount;
  long picker, tcount;
  /*********************************************************************
   *
   * funcs returns the m basis functions in a[] evaluated at x 
   * degree of freedom array set with one degree of freedom
   */
  //
  void funcs(double *, double [], int, int );
  //*********************************************************************
   

  // Open the files.
  datout.open(outfile);
  cellout.open(cellfile) ; 
  if (exportCovariance) covout.open(CovarFile);

  // Define the set of arbitrary grids that we will use
  NXGRIDS = (I->xsize - 2*(FLAGDIST)) / (XBUFFERSIZE - 2*(FLAGDIST)) + 1;
  NYGRIDS = (I->ysize - 2*(FLAGDIST)) / (YBUFFERSIZE - 2*(FLAGDIST)) + 1;

  cout    << "NXGRIDS, NYGRIDS = " << NXGRIDS << "," << NYGRIDS << endl;
  cellout << "#NXGRIDS, NYGRIDS = " << NXGRIDS << "," << NYGRIDS << endl;

  // Write headers
  if (library->fluxnorm) { 
    datout << "# Normalized templates used: FitQuantity = Flux" << endl;
  } else {
    datout << "# Templates not normalized: FitQuantity = Scale Factor" << endl;
  }

  //  datout << "#ObjID      X       Y    Cell     cx      cy     Rcell       FitQty     FitQuErr   SExF     Totflux " << endl;
  //  cellout << "#Cell Nobj    NX    NY     LX    LY    lx0   ly0 Background  BGErr     Chi2         DoF     Chi2/DoF     new_npix" << endl;

  datout << "#  1   ObjectID" << endl;
  datout << "#  2   X" << endl;
  datout << "#  3   Y" << endl;
  datout << "#  4   Cell "  << endl;
  datout << "#  5   cx          X pos in cell "  << endl;
  datout << "#  6   cy          Y pos in cell "  << endl;
  datout << "#  7   Rcell "  << endl;
  datout << "#  8   FitQty      Fit Quantity (Flux or Scale Factor)"  << endl;
  datout << "#  9   FitQuErr    Variance of Fit Quantity"  << endl;
  datout << "# 10   ProjErr     Chi2ellipse projected onto object axis"<< endl;
  datout << "# 11   SExF        SExtractor flux in hires catalog"  << endl;
  datout << "# 12   Totflux     Total flux in hires cutout" << endl;

  cellout << "#  1   CellID " << endl;
  cellout << "#  2   Nobj        Num objects in cell " << endl;
  cellout << "#  3   NX          XDim of cell " << endl;
  cellout << "#  4   NY          YDim of cell " << endl;
  cellout << "#  5   LX          X pos of lower left corner of cell " << endl;
  cellout << "#  6   LY          Y pos of lower left corner of cell " << endl;
  cellout << "#  7   lx0         X pos of original (ungrown) llc " << endl;
  cellout << "#  8   ly0         Y pos of original (ungrown) llc " << endl;
  cellout << "#  9   Background  Background value for this cell " << endl;
  cellout << "# 10   BGErr       Error in background value " << endl;
  cellout << "# 11   Chi2        Chi-squared for this cell solution " << endl;
  cellout << "# 12   DoF         Degrees of freedom for this cell " << endl;
  cellout << "# 13   Chi2/DoF    Chi-squared per degree of freedom " << endl;
  cellout << "# 14   new_npix    Npix used in fit (after cellmask applied) " << endl;


  // Walk over each grid spacing: Why does this start from 1??? Image pixels
  // start from zero.
  
  //8 Apr 05: Added a third loop for a half-cell-size dither offset
  //12 Apr 05: Made the dither by half the distance between cell centers.
  //19 May 05: Made whether to dither a user parameter

  xditherwidth = (XBUFFERSIZE - 2*FLAGDIST)/2;
  yditherwidth = (YBUFFERSIZE - 2*FLAGDIST)/2;
  cout << "x/y ditherwidth = " << xditherwidth << "," << yditherwidth << endl;
  for (dither=0;dither<=dithercell;dither++)   
   for (myi=1;myi<=NXGRIDS;myi++) 
     for (myj=1;myj<=NYGRIDS;myj++) {


      GRIDCOUNT++;

      cout << "~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~" << endl;
      cout << "Beginning buffer "<< GRIDCOUNT << ": " << endl;
      cout << "Dither, dithercell = " << dither <<","<<dithercell<<endl;

      // Compute the bounds of this cell...
      XMIN = XBUFFERSIZE*(myi-1) - 2*(myi-1)*FLAGDIST + dither*xditherwidth;
      XMAX = XBUFFERSIZE*(myi)   - 2*(myi-1)*FLAGDIST + dither*xditherwidth;
      YMIN = YBUFFERSIZE*(myj-1) - 2*(myj-1)*FLAGDIST + dither*yditherwidth;
      YMAX = YBUFFERSIZE*(myj)   - 2*(myj-1)*FLAGDIST + dither*yditherwidth;
      //.. ensuring that we don't go outside the image.
      XMAX = (XMAX > I->xmax) ? I->xmax : XMAX;
      YMAX = (YMAX > I->ymax) ? I->ymax : YMAX;

      XCEN = (XMAX - XMIN)/2.0;
      YCEN = (YMAX - YMIN)/2.0;

      cell.Set(XMIN, YMIN, XMAX, YMAX);
      newcell = cell;

      cout << "Lx0, Ly0 = " << XMIN << "," << YMIN << endl;

      // Find the range of cells in the grid that we care about.
      lowcell  = BigGrid->Contains(XMIN,YMIN);
      highcell = BigGrid->Contains(XMAX,YMAX);

      //Compute the number of objects there will be.
      //crowdsize = 0; replaced by crowd.size()
      crowd.clear();
      growcount=0;
      for(m=lowcell[XX];m<=highcell[XX];m++)      // xcoords 
	for(n=lowcell[YY];n<=highcell[YY];n++) {  //ycoords
	  item = BigGrid->grid[n][m]->Top();
	  // Loop through the list for this grid cell
          while (item != NULL) {
	     // Make sure the object center is in the cell
	    item->getCoords(&obx,&oby);
	    if (cell.Contains(obx,oby)){
	      cout << "Cell contains " << obx <<" "<< oby << endl;
	       // If so, check its template bounds
	       if (cell.Contains(*item->Range() ) ){
		 crowd.push_back(item);
	       }
	       else if (cell.IsInside(*item->Range() ) || 
			cell.Overlaps(*item->Range())   ) {
		 crowd.push_back(item);
		 cout << "...Growing to hold object #" <<item->Num()<<endl;
		 newcell.GrowsToHold(*item->Range());
		 growcount++;
	       }
	     } //end if object is in cell
	     else {
	       cout << "object "<< item->Num() <<" discarded from square "
		      << m<< "," << n << endl;
	     } // end else object is not in cell
          item = BigGrid->grid[n][m]->Next(item);
	  }

	}	
    
		  
		  

      if (crowd.empty() ) {
         cout << "Skipping buffer "<< GRIDCOUNT <<": Zero objects " << endl;
      } else {
	cout <<  "... "<<crowd.size()<<" objects."<<endl;
	cout << growcount <<" objects or "<<100.0*float(growcount)/crowd.size() << "% extended past original cell boundaries." << endl;
	//.............Method not yet written.....................
	// Make sure we didn't overstep the bounds of the image.
	//	if (not I->xyrange->Contains(newcell) {
	//  cout << "Shrinking to fit inside image" << endl;
        //  newcell.ShrinksToFit(I);
	//}
	//........................................................

	// Create the image vector that will hold each lowres cell
	image = dvector(1,newcell.Npix() );
	err   = dvector(1,newcell.Npix() );
	imflag= lvector(1,newcell.Npix() );

	//Grab the right section of image & weight map, and squash them into a vector..
	I->Flatten(newcell, image);
        E->Flatten(newcell, err);	
	F->Flatten(newcell, imflag);

	//

	  // For each object in the relevant grid cells, 
	  // get the template image & 
	  // put it into a properly sized "flattened frame"
          crowdsize=crowd.size();
	  temp = dmatrix(1,newcell.Npix(),1,crowd.size());
	  if (temp == NULL) {cout << "error allocating matrix" << endl;} //should stop
	  pcounter = 0;
// 	  for (i=1; i<=newcell.Npix(); i++) 
// 	    for (j=1; j<=crowd.size(); j++) {
	  for (j=1; j<=newcell.Npix(); j++) 
	    for (i=1; i<=crowd.size(); i++) {
               temp[j][i] = 0.0;
               pcounter++;
	    }

          //tcount = crowd.begin();
	  //while (tcount !=crowd.end()) {

	  int ii,jj;
	  cout << "ID & cell coords of template (lx,ly)" << endl;
	  for (picker=0; picker<crowdsize; picker++) {
		// Read the image from the library
            item=crowd[picker];
            thisone = library->Get(item->Num());
	    for (k=1; k<newcell.Npix(); k++) {temp[k][picker+1] = 0.0; }
	    thisone->Flatten(newcell, temp, picker+1);
	    ii = thisone->xmin + thisone->xyrange.Lx() - newcell.Lx();
	    jj = thisone->ymin + thisone->xyrange.Ly() - newcell.Ly();
	    cout << item->Num() << " " << ii << " " << jj << endl;

	    
	    //  tcount++;
	  }

	  //------------------------------------------------------------------------------
	  // New attempt to mask out the pixels that are irrelevant to this solution.
	  if (use_cellmask) {
	    mask = cellmask(mask_floor, max_flag, imflag, temp, 
			    crowdsize, newcell.Npix(), new_npix);
	    newtemp = dmatrix(1,new_npix,1,crowd.size());
	    newimage = dvector(1,new_npix);
	    newerr = dvector(1,new_npix);
	    // Only take the pixels that are not masked out
	    k=1;
	    for (j=1; j<=newcell.Npix(); j++){
	      if (mask[j] > 0) {
		newimage[k]=image[j];
		newerr[k]=err[j];
		for (i=1; i<=crowd.size(); i++) {newtemp[k][i]=temp[j][i];}
		k++;
	      }
	    }
	  }
	    /* If there's no problem with arg passing, do this:
	       ...........................................................................
	       tfit_engine(npix, crowdsize, image, temp, err, //[input]
	       dof, covar, &chi2) ; //[output]

	       // Output results for each object: Rework for lack of Crowd
	       pnt = Crowd->Top();
	       while (pnt != NULL) {
	       totflux += pnt->getISOFlux() * dof->data[pnt->getGroup()+1];
	       totchi2 += chi2;
	       tfit_output(pnt, datout, covout, dof, covar, &chi2);
	       pnt = Crowd->Next(pnt);
	       ...........................................................................
	    */

	    // Set up the degrees of freedom for the minimization
	    // If sky background turned off, do it here: dof->usedata[1]=0
	    // ... but usedata is not used, so it will actually be harder than that.
	  
	  dof = new Par(crowdsize+fit_to_bg);
	  covar = dmatrix(1,dof->size,1,dof->size);
	  if (use_cellmask) {
	    svd_U = dmatrix(1,new_npix,1,dof->size); 
	    } else {
	    svd_U = dmatrix(1,newcell.Npix(),1,dof->size); 
	    }
	  svd_V = dmatrix(1,dof->size,1,dof->size);
	  svd_W = dvector(1,dof->size);

	    // initialize covar
	  pcounter=0;
	  for (i=1; i<=dof->size; i++)
	    for (j=1; j<=dof->size; j++) {
	      covar[j][i]=0.0;
	      pcounter++;
	    }

	    /* Minimization: */

#ifndef LFIT

	  if (use_cellmask) {
	  svdfit(newtemp,newimage,newerr,new_npix,dof->data,
		 dof->size,svd_U,svd_V,svd_W,&chi2,&funcs,fit_to_bg);
	  } else {
	  svdfit(temp,image,err,newcell.Npix(),dof->data,
		 dof->size,svd_U,svd_V,svd_W,&chi2,&funcs,fit_to_bg);
	  }
	  svdvar(svd_V,dof->size,svd_W,covar);
#endif /* SVD */

#ifdef LFIT
	  dofflag = ivector(1,dof->size);
	  for (i=1;i<=dof->size;i++) dofflag[i] = 1;
	  //lfit(temp,image,err,newcell.Npix(),dof->data, 
	     lfit(newtemp,newimage,newerr,new_npix,dof->data, 
	       dofflag, dof->size, covar, &chi2, &funcs);
	  free_ivector(dofflag,1,dof->size);
#endif /* LFIT */

	  totchi2 += chi2;
	    /************** output stuff starts here***********************/
	    /* Write results to output */
	  
	    datout << setprecision(3); datout.setf( ios::fixed );

	    if (exportCovariance) {
	      covout << endl << "#Cell id: " << GRIDCOUNT << endl;
	      covout << "#Arraydim: " << crowdsize+1 << " " << crowdsize+fit_to_bg << endl;
	      // First column is the object id. Use 0.0 for sky.

	      covout << "#Fitted flux follows .. " << endl;
              for (j=1;j<=dof->size;j++)
		covout << dof->data[j] << " ";
	      covout << endl;
	      covout << " " << endl;

		       //write the w's too
	      covout << "#1/W follows .. " << endl;
	      for (j=1;j<=dof->size;j++)
		covout << svd_W[j] << " ";
	      covout << endl;
	      covout << endl;


	      //covout << "Placeholder? : 0" << endl;
	      //covout << "#covar[1][*].." << endl;

	      //Starts with the background row.
	      covout << "#Covar matrix follows... " << endl;
	      if (fit_to_bg) {
		covout << "0 ";
		for (j=1;j<=dof->size;j++)
		  covout << covar[1][j] << " ";
		covout << endl;
	      }



	    }
            tcount = fit_to_bg;  //There's an offset in dof->data & covar, because of sky.

            for (picker=0; picker<crowdsize; picker++) {
	      //for(m=lowcell[XX];m<=highcell[XX];m++)      // xcoords 
	      //for(n=lowcell[YY];n<=highcell[YY];n++) {  //ycoords
		// Loop through each grid cell
	        //pnt = BigGrid->grid[m][n]->Top();
		// Loop through the list for this grid cell
                     pnt = crowd[picker]; 
                     tcount++;
            	     datout << setw(6) << pnt->Num() << " " ; //which object



		     //	      datout << tcount << " ";
		     datout.setf(ios::fixed,ios::floatfield | ios::showpoint);
		     pnt->getCoords(&xx,&yy); //x,y

		     datout << setprecision(2); 
		     datout << setw(8) << xx << " " << setw(8) << yy << " ";


		     //Floating cell number & distance from cell center.
                     
		     datout << setw(4) << GRIDCOUNT << " " ;
		//     datout << newcell.Xdim() <<  " " << newcell.Ydim() << " ";
		     //     datout << " " << xx-newcell.Lx() << " " << yy-newcell.Ly() << " " ;

		     xc = newcell.Xpos(xx);
		     yc = newcell.Ypos(yy);
		     //xc = int(xx-newcell.Lx());
		     //yc = int(yy-newcell.Ly());
		     datout << setprecision(2);
		     datout << setw(8) << xc << " " << setw(8) << yc << " ";
		     // rcell = sqrt( (XCEN-xc)*(XCEN-xc) + (YCEN-yc)*(YCEN-yc) );

		     rcell = newcell.RadialDistance(xx,yy);

// 		     cx = int(xx-newcell.Lx()); //int'd
// 		     cy = int(yy-newcell.Ly());
// 		     rcell = sqrt( (XCEN-cx)*(XCEN-cx) + (YCEN-cy)*(YCEN-cy) );
		     datout << setprecision(4); 
		     //	     datout.setf(0, ios::floatfield);

		     datout << setw(8) << rcell << " ";

// 		     datout << dof->data[1] << " "; // background
// 		     datout << sqrt(covar[1][1]) << " "; //error

		     if (use_cellmask){
		       oberr=sqrt(covar[tcount][tcount]); //* sqrt(chi2/(new_npix - dof->size));
		       } else {
			 oberr=sqrt(covar[tcount][tcount]); // * sqrt(chi2/(newcell.Npix() - dof->size));
			 }
		     datout << setw(12) << dof->data[tcount] << " "; //scale factor
		     datout << setw(12) << oberr << " "; //error

		     // Chi2 ellipse projected onto this object
		     maxcomp=getmaxcomp(svd_V,crowdsize+fit_to_bg,tcount);
		     datout << setw(12) << fabs(maxcomp/svd_W[tcount]) << " ";
		     
		     //
		     //sqrt(covar[i+1][i+1]

		     //Object properties: fluxA, fluxB
		     //datout.setf(0,ios::floatfield | ios::showpoint);
		     datout.setf(ios::floatfield | ios::showpoint);
		     datout << setw(10) << pnt->getSExFlux() << " " << setw(10) << pnt->getISOFlux();

//...............................................................
//  Remove these last two columns because they would have to vary
// according to what the FitQuantity is (flux or scale factor)
//...............................................................
// 		     //datout.setf(0, ios::floatfield | ios::showpoint); 
// 		     datout.setf(ios::floatfield | ios::showpoint); 
// 		     datout << setprecision(4) ;
// 		     datout << setw(12) << pnt->getISOFlux()*dof->data[tcount];

// 		     //errors: flux*object error = uncertainty on the scaled flux

// 		     if (USESIG) datout << setw(12) << pnt->getISOFlux() * 
// 				   sqrt(covar[tcount][tcount]);
// 		     // crazy estimated guess of errors
// 		     else datout << setw(12) << pnt->getISOFlux() * 
// 			    sqrt(covar[tcount][tcount]) *
// 			    sqrt(chi2/(newcell.Npix() - dof->size));
//..........................................................................
// 		     // chi2
// 		     datout.setf(0,ios::floatfield);
// 		     datout << " " << chi2; 
// 		     // # degrees of freedom
// 		     datout << " " << (newcell.Npix() - dof->size);
		     
		     datout << endl;

		     if (exportCovariance) { //probably we will set this off usually
		       covout << pnt->Num()<< " " ; //<< endl;
		       // covout << "..covar[tcount][*].." << endl;
		       for (j=1;j<=dof->size;j++)
			 covout << covar[tcount][j] << " ";
		       covout << endl;
		     }

		     // Collect up total flux: only once per object.
		     if (pnt->isFlagged() == 1) {
		       totflux += pnt->getISOFlux() * dof->data[tcount];
		     }

		 // end list in this grid cell

	     
	      } // end looping over crowd

	      // Write the V array too
	      if (exportCovariance) {
		covout << endl;
		covout << "#V follows .. " << endl;
		for (tcount=1; tcount<=dof->size; tcount++){
		  for (j=1; j<=dof->size;j++){
		    covout << svd_V[tcount][j] << " ";
		  }
		  covout << endl;
		}
		covout << endl;
		covout << endl;
	      }

	      // Write cell stats for this cell
	      cellout << setw(4) << GRIDCOUNT << setw(4) << crowdsize << " " ;
	      cellout << setw(6) << newcell.Xdim() << " " <<  setw(6) << newcell.Ydim() << " ";
	      cellout << setw(6) << newcell.Lx() << " " << setw(6) << newcell.Ly() << " ";
	      cellout << setw(6) << XMIN << " " << setw(6) << YMIN << " ";
	      cellout << setprecision(4); 
	      //cellout.setf(0, ios::floatfield | ios::showpoint);
	      cellout.setf(ios::floatfield | ios::showpoint);
	      if (fit_to_bg == 1) {
		cellout << setw(8) << dof->data[1] << " "; // background
		cellout << setw(8) << sqrt(covar[1][1]) << " "; //error
	      } else {
		cellout << "...Not Fitted..." << " ";
	      }
	      // chi2
	      //cellout.setf(0,ios::floatfield | ios::showpoint);
	      cellout.setf(ios::floatfield | ios::showpoint);
	      cellout << setw(10) << chi2 << " "; 
	      // # degrees of freedom
	      // chi2/dof
	      if (use_cellmask) {		
		cellout << setw(10) << (new_npix - dof->size) << " ";	      
		cellout << setw(10) << chi2/(new_npix - dof->size) << " ";
              } else {
		cellout << setw(10) << (newcell.Npix() - dof->size) << " ";
	      
		cellout << setw(10) << chi2/(newcell.Npix() - dof->size) << " ";
	      }
	      cellout << setw(7) << new_npix;
	      cellout << endl;


	    // Memory freeing starts here..................................
	
            free_dmatrix(temp,1,newcell.Npix(),1,crowdsize);

	    free_dmatrix(svd_U,1,newcell.Npix(),1,dof->size);
	    free_dmatrix(covar,1,dof->size,1,dof->size);
	    free_dmatrix(svd_V,1,dof->size,1,dof->size);
	    free_dvector(svd_W,1,dof->size);
  
            free_dvector(image,1,newcell.Npix());
            free_dvector(err,1,newcell.Npix());
	    free_lvector(imflag,1,newcell.Npix());

            free_ivector(lowcell,1,2);
            free_ivector(highcell,1,2);

	    if (use_cellmask) {
              free_dmatrix(newtemp,1,new_npix,1,crowdsize);
	      free_dvector(newimage,1,new_npix);
	      free_dvector(newerr,1,new_npix);
	      free_ivector(mask,1,newcell.Npix());
	    }

  
	  } // Finish check that floating cell has objects

	} // Finish looping over grids and dither pattern


  // Begin final cleanup here

  if (exportCovariance) covout.close();
  datout.close();
  cellout.close();
  //  cout.precision(10);
  //  cout << "Total Chi^2 of All Fitted Objects: " << totchi2 << endl;
  //  cout << "Total Flux of All Fitted Objects:  " << totflux << endl;

  /*********************************************************************
    *
    * Clean up objects - will have to do this earlier?
    *
    *********************************************************************/
  


//   delete pnt;  Didn't allocate it; don't deallocate it here.
  //  delete BigGrid;  Do this in main.
 
}

 
 
