/*********************************************************************
 * main.cc
 * Created 1 Feb 1999 - Papovich
 *   This program matches the given isophotes for "perfect"
 *   resolution to the same image with poorer resolution.
 * * *
 * Major revision: Oct 25 2002 - Laidler
 *  Revised architecture for efficiency. Template images are now
 * convolved and resampled to low resolution in an earlier processing
 * step. TFIT now reads this library of templates, instead of producing
 * them on the fly. 
 *
 * * *
 * Revision to add new parameters - Laidler
 *   Dither parameter to control whether cell pattern is dithered
 *   Background subtraction parameter to subtract background in memory.
 *
 * * *
 * Revision Mar 23 2006 - Laidler
 *     Version renumbering in define.h in preparation for first
 * public release: Current version redefined as version 1.0beta.
 * * *
 * Revision Dec 20 2007 - Laidler
 *     Version renumbering in define.h for 1.0 release.
 *********************************************************************/

#include <iostream.h>
#include <iomanip.h>
#include <math.h>
#include <fstream.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <assert.h>
#include <time.h>
#include "fitsio.h"
#include "nrutil.h"
#include "define.h"
#include "fits.h"
#include "fitsmanip.h"
#include "structs.h"
#include "funcs.h"
#include <vector>
#include "tlibrary.h"
#define XGRID_SIZE Image->xsize / 20 // Make these arguments soon
#define YGRID_SIZE Image->ysize / 20 // was 10: changed to 20 for debug

int main(int argc, char *argv[]) {

  /*********************************************************************
   *
   * DEFINE & DECLARE VARIABLES
   *
   *********************************************************************/
  int 
    i, j,
    status=0,
    line=0,
    nosub=0,
    MAX_FLAG=0,
    USE_ERRORS=FALSE,
    USE_CELLMASK=FALSE,
    DITHERCELL=FALSE,
    CREATE_TEMPLATE=FALSE,
    CHECK_TEMPLATES=FALSE,
    EXPORT_COV_INFO=FALSE,
    FIT_BACKGROUND=TRUE,
    SETGROUP=TRUE,
    XBUFFERSIZE=DEFAULT_BUFFER,
    YBUFFERSIZE=DEFAULT_BUFFER,
    FLAGDISTANCE=DEFAULT_FLAG_DIST;
  float 
    RMS_CONSTANT=1.0,
    BG_CONSTANT=0.0,
    secs,
    MASK_FLOOR=0.0;
  time_t 
    start=time(NULL),
    end;
  Fits 
    *Image, 
    *thisone,
    *FlagMap,
    *WeightMap;  
  tLibrary
    *Library;

  fitsfile 
    fimage, 
    ferr;
  Funcs masterCode;   
  FitsManip fitsmanip;
  ifstream parin;

  Grid * BigGrid;
  long * objlist;


  char *token;
  char CHOICE[STRING_LENGTH],
    SexFile[STRING_LENGTH], 
    LoResFits[STRING_LENGTH],
    TemplateFits[STRING_LENGTH],
    WeightFits[STRING_LENGTH], 
    FlagFits[STRING_LENGTH], 
    ConSegFits[STRING_LENGTH], 
    GrpFits[STRING_LENGTH], 
    growthfile[STRING_LENGTH], 
    convfile[STRING_LENGTH],
    datafile[STRING_LENGTH], 
    cellfile[STRING_LENGTH], 
    TemplateImageFile[STRING_LENGTH], 
    TemplateDir[STRING_LENGTH],
    ResidualImageFile[STRING_LENGTH], 
    CovarianceFile[STRING_LENGTH],
    errchoice[STRING_LENGTH];

  void printTime(float);

  strcpy(SexFile,DEFAULT_SEXFILE);
  strcpy(LoResFits,DEFAULT_LORES);
  strcpy(TemplateFits,DEFAULT_TEMPLATE);
  strcpy(WeightFits,DEFAULT_WEIGHTFITS);
  strcpy(FlagFits,DEFAULT_FLAGMAP);
  strcpy(ConSegFits,DEFAULT_CONSEGFITS);
  strcpy(growthfile,DEFAULT_GROWTHFILE);
  strcpy(convfile,DEFAULT_CONVFILE);
  strcpy(GrpFits,DEFAULT_GRPFITS);
  strcpy(datafile,DEFAULT_DATAFILE);
  strcpy(cellfile,DEFAULT_CELLFILE);
  strcpy(TemplateImageFile,DEFAULT_TEMPLATEFILE);
  strcpy(ResidualImageFile,DEFAULT_RESIDUALFILE);
  strcpy(CovarianceFile,DEFAULT_COVFILE);

  cout << "*** TFIT: fitter version " << VERSION << " ***" << endl;
  if (argc < 2) {cout << "Usage: tfit <parameter file> "<< endl; return -1;}

  /*********************************************************************
   *
   * List of input files and parameters: 
   *  (lines starting with '#' in the param fil
   *  are ignored).
   *  --------------------
   *  SexFile : the output list from SExtractor (or something similar)
   *  LoResFits : the Low Resolution image (.fits) image to be chi2 fitted
   *  Templatefits : the Hi-Res image (.fits) resolution fits image
   *  WeightFits : The weight map for the LoRes image (.fits) image
   *  SegImgFits : The segmentation map (.fits) image of the 
   *               Hi-res image (PIXTYPE=INT)
   *  datafile : output file -- will contain fluxes, scaling factors, etc.
   *  convfile : the input file holding the convolution kernel 
   *  USE_ERRORS: if true, then the routine uses the weight map -- 
   *      else no weight used and approximate errors will be given for the 
   *      final fluxes.
    *  RMS_CONSTANT:  A constant which gives the errors from the weight map
   *      via the relation:  err = RMS_CONSTANT / weight
   *  EXPORT_COVAR_INFO: The program will write all the covariance matrix in
   *      formation for each group fitted to a file, where the file is
   *	 specified by:
   *  COVARIANCE_FILE: (see above)
   *
   *********************************************************************/
  
  /*********************************************************************
   *
   * READ IN PARAMETER FILE
   * (Reading in flags from command line is no longer supported.)
   *********************************************************************/
  parin.open(argv[1]);
  if (!parin) {
    cout << "unable to find param file\n";
    return 1;
  }
  cout << "Reading parameter file: " << argv[1] << endl;
  parin.setf(ios::uppercase);
  while(parin) {
    parin.getline(CHOICE,LINE_SIZE); line++;
    if (!strcmp(CHOICE,"")) break;
    while (!strncmp(CHOICE,"#",1)) {
      parin.getline(CHOICE,LINE_SIZE); line++;}
    // the above statement tells the routine to ignore lines starting with '#'
    token = strtok(CHOICE,",;\t \n"); 
    if (!strncmp(token,"OBJECT",6)) 
      strcpy(SexFile,token=strtok('\0',",;\t "));
    else if (!strncmp(token,"LOW_RES",7)) 
      strcpy(LoResFits,token=strtok('\0',",;\t "));
    else if (!strncmp(token,"FLAG_MAP",8)) 
      strcpy(FlagFits,token=strtok('\0',",;\t "));

    else if (!strncmp(token,"OUTPUT",6)) 
      strcpy(datafile,token=strtok('\0',",;\t "));
    else if (!strncmp(token,"CELLFILE",8)) 
      strcpy(cellfile,token=strtok('\0',",;\t "));

    else if (!strncmp(token,"USE_ERR",7)) 
      strcpy(errchoice,token=strtok('\0',":,;\t "));
   else if (!strncmp(token,"ERR_MAP",6)) 
      strcpy(WeightFits,token=strtok('\0',",;\t "));
    else if (!strncmp(token,"RMS_CON",7)) 
      RMS_CONSTANT = atof(token=strtok('\0',",;\t "));

    else if (!strncmp(token,"FIT_BACK",8)) {
      if (atoi(token=strtok('\0',",;\t ")) == 1 || 
	  !strncmp(token,"T",1) ||
	  !strncmp(token,"Y",1)) FIT_BACKGROUND = TRUE;
      else FIT_BACKGROUND = FALSE;
    }

    else if (!strncmp(token,"BG_CON",6)) 
      BG_CONSTANT = atof(token=strtok('\0',",;\t "));

    else if (!strncmp(token,"CELLMASK",8)) {
      if (atoi(token=strtok('\0',",;\t ")) == 1 || 
	  !strncmp(token,"T",1) ||
	  !strncmp(token,"Y",1)) USE_CELLMASK = TRUE;
      else USE_CELLMASK = FALSE;
    }
    else if (!strncmp(token,"FLOOR",5)) 
      MASK_FLOOR = atof(token=strtok('\0',",;\t "));

    else if (!strncmp(token,"DITHERCELL",10)) {
      if (atoi(token=strtok('\0',",;\t ")) == 1 || 
	  !strncmp(token,"T",1) ||
	  !strncmp(token,"Y",1)) DITHERCELL = TRUE;
      else DITHERCELL = FALSE;
    }

    else if (!strncmp(token,"MAX_FLAG",8)) 
      MAX_FLAG = atoi(token=strtok('\0',",;\t "));


    else if (!strncmp(token,"TEMPLATE_",9)) 
      strcpy(TemplateDir,token=strtok('\0',",;\t "));
    else if (!strncmp(token,"EXPORT_",7))  {
      if (atoi(token=strtok('\0',",;\t ")) == 1 || 
	  !strncmp(token,"T",1) ||
	  !strncmp(token,"Y",1))  EXPORT_COV_INFO=TRUE;
      else EXPORT_COV_INFO = FALSE;
    } 

    else if (!strncmp(token,"COVAR",5)) 
      strcpy(CovarianceFile,token=strtok('\0',",;\t "));
    else if (!strncmp(token,"XBUFFER",7))
      XBUFFERSIZE=atoi(token=strtok('\0',",;\t "));
    else if (!strncmp(token,"YBUFFER",7))
      YBUFFERSIZE=atoi(token=strtok('\0',",;\t "));
    else if (!strncmp(token,"FLAG_DIST",9))
      FLAGDISTANCE=atoi(token=strtok('\0',",;\t "));
    else {
      cout << "% KEYWORD: " << token << " unknown in line: ";
      cout << line << endl;
    }
  }
  parin.close();

  /********************************************************************
   * 
   * Reading in flags from command line is no longer supported.:
   *
   ********************************************************************/

  /*********************************************************************
   *
   * READ IN FITS FILES
   *
   *********************************************************************/

  cout << "Opening LowRes image..." << endl;
  Image = fitsmanip.ReadFits(&fimage,LoResFits,&status,"Lo Res Image",nosub);
  if (BG_CONSTANT != 0) {
    cout << "Subtracting BG_CONSTANT " << BG_CONSTANT << " from image" << endl;
    Image->subtract(BG_CONSTANT);
  }

  cout << "Reading object catalog..." << endl;
  // Populate the BigGrid from the hires SExtractor catalog
  BigGrid = new Grid(SexFile,(long)Image->xsize,(long)Image->ysize,(long)XGRID_SIZE,
		     (long)YGRID_SIZE);

 //    for (i=1; i<11; i++)
//     for (j=1; j<11; j++) {
//       cout << "Grid cell " << j << "," << i << ": Size " << BigGrid->grid[j][i]->Size() << endl;
//     }

  BigGrid->list();  // Diagnostic output
  cout << "Reading template library..." << endl;
  Library = new tLibrary(BigGrid, TemplateDir, Image->crpix1, Image->crpix2);
  Library->setRanges(BigGrid);

  if (!strncmp(errchoice,"WEIGHT",6) || !strncmp(errchoice,"RMS",3)) {
    USE_ERRORS = TRUE;
    cout << "Reading error map ..." << endl;
    WeightMap = fitsmanip.ReadFits(&ferr,WeightFits,&status,"Weight Map", nosub);
    // If weight map, need to transform to rms
    if (!strncmp(errchoice,"WEIGHT",6)) {
      cout << "     Converting weight map to RMS map via RMS constant = "  << RMS_CONSTANT << endl;
      for (i=WeightMap->xmin; i<=WeightMap->xmax; i++)
	for (j=WeightMap->ymin; j<=WeightMap->ymax; j++) {

	  if (WeightMap->data[j][i] == 0.0) {
	    WeightMap->data[j][i] = MAX_FLOAT;
	  } else {
	   WeightMap->data[j][i] = RMS_CONSTANT/sqrt(WeightMap->data[j][i]);
	  }
	}
    } else { cout << "     Using RMS map exactly as provided" << endl; }
  } else {
      WeightMap = new Fits(Image->xmin, Image->xmax, 
                           Image->ymin, Image->ymax, TFLOAT, RMS_CONSTANT);
      if (!strncmp(errchoice,"NONE",4)){
	WeightMap->newTitle("Null Weight Map");
	cout << "     Using uniform RMS map with value " << RMS_CONSTANT << endl;
      } else { 
	WeightMap->newTitle("Scalar Weight Map");
        cout << "     Using uniform RMS map with value 1" << endl;
      }
  }

  if (!strncmp(FlagFits,"NONE",4)){
    cout << "Opening flag image..." << endl;
    FlagMap = fitsmanip.ReadFits(&fimage,FlagFits,&status,"Flag Map",nosub);
  } else {
    cout << "No flag image used; setting max_flag = 0 " <<endl;
    FlagMap = new Fits(Image->xmin, Image->xmax, 
                       Image->ymin, Image->ymax, TLONG, 0);
    FlagMap->newTitle("Null flag map");
    MAX_FLAG = 0;
  }
  /*********************************************************************
   *
   * Start the main body of the code
   *
   *********************************************************************/

   masterCode.walkGrid(Image,WeightMap,FlagMap,Library,USE_ERRORS,
		     datafile, cellfile, BigGrid,
		       RMS_CONSTANT, 
                     USE_CELLMASK, MASK_FLOOR, DITHERCELL, 
		     MAX_FLAG, FIT_BACKGROUND,
		     EXPORT_COV_INFO,CovarianceFile,
		     XBUFFERSIZE,YBUFFERSIZE,FLAGDISTANCE,
  		     &status);
   // Clean up & go home.
  delete Image;
  delete FlagMap;
  // Should delete Library; but no destructor written

  if (USE_ERRORS) delete WeightMap;
  delete BigGrid;  //Mystery problem: must delete grid after weightmap.

  end = time(NULL);
  secs = difftime(end,start);

  printTime(secs);
  cout << "TFIT Done...FITS status = " << status << endl;
  return(status);
}

/*********************************************************************
 *
 * SUBROUTINES
 *
 *********************************************************************/

void printTime(float secs) {
  int hours=0,mins=0;
  if (secs >=60) {
    mins = (int) floor(secs) / 60;
    secs -= 60.0*(float)mins;
  }
  if (mins >=60) {
    hours = mins/60;
    mins -= 60*hours;
  }
  cout << "Runtime = ";
  if (hours) cout << hours << "h:" << mins << "m:" << secs <<"s" << endl;
  else if (mins) cout << mins << "m:" << secs << "s" << endl;
  else cout << secs << "s" << endl;
  return;
}

