#include <iostream.h> 
#include <iomanip.h>
#include "fitsio.h"
#include "fits.h"
#include "fitsmanip.h"
#include <glob.h>
#include "structs.h"

#include "tlibrary.h"

tLibrary::tLibrary() {
  title="Empty Library";
  size = 0; smart=false;
}

tLibrary::tLibrary(Grid* BigGrid, char* directory, 
		   float big_crpix1, float big_crpix2) {
  fitsfile fimage;
  FitsManip fitsmanip;
  int k, status;
  char filename[200];
  char* num; // to turn numbers into
  long * objlist;
  int subimage = 1;  //Sets the xyrange coord system according to CRPIX
  float local_crpix1, local_crpix2;

  objlist = BigGrid->objlist();

  size = 0; smart = false;
  for (k=0;k<BigGrid->count;k++) {
    sprintf(filename,"%s%d%s",directory,objlist[k],".fits");
    //    filename=strcat(directory,num);
    //filename=strcat(filename,".fits");
    shelf[objlist[k]]=fitsmanip.ReadFits(&fimage, filename, &status,
                             filename, subimage);
    local_crpix1=shelf[objlist[k]]->crpix1;
    local_crpix2=shelf[objlist[k]]->crpix2;
    shelf[objlist[k]]->xyrange.Move(big_crpix1-local_crpix1,
				    big_crpix2-local_crpix2);
  
    //  cout << "Title: " << shelf[objlist[k]]->Title() <<endl; //<< " Filename: " filename << endl;
      size++;   //warning, no error checking!
  }
  // See if these are normalized templates, by checking 1 file.
  fluxnorm = fitsmanip.CheckNorm(&fimage, filename, &status);

  free(objlist);
}

tLibrary::tLibrary(char* directory) {
  // Note that I am using some Posix routines here, opendir and readdir.

  //  DIR *opendir(const char*), *dirstream;
  //  struct dirent *readdir(DIR *), *dir_entry, filename;
  char *pattern;
  glob_t dirlist;
  int gflags=0, *gerr=NULL;

  fitsfile fimage;
  FitsManip fitsmanip;
  int status;
  int subimage = 1;  //Sets the xyrange coord system according to CRPIX

  size = 0; smart = false;
  // Open the directory.
  // dirstream = opendir(directory);
  pattern = strcat(directory,"/*.fits");
  status = glob(pattern, gflags, NULL, &dirlist);
  //status = glob("../data/tmplib/*.fits", gflags, NULL, &dirlist);
  if ((status == 0) && (dirlist.gl_pathc > 0)) {
    // See if these are normalized templates, by checking 1 file.
    fluxnorm = fitsmanip.CheckNorm(&fimage, &dirlist.gl_pathv[0][0], &status);
    // For every file in the directory,
    while (dirlist.gl_pathv[size] != NULL) {
      // Add the Fits object to the shelf.
      // shelf.push_back(fitsmanip.ReadFits(fimage, dir_entry->d_name, &status, 
      //                                      dir_entry->d_name));
      
     shelf[size]=fitsmanip.ReadFits(&fimage, &dirlist.gl_pathv[size][0], &status,
                             &dirlist.gl_pathv[size][0], subimage);

      size++;   //warning, no error checking!

      } // Read all objects into shelf
  }
  title = directory;
  globfree(&dirlist);

}

Fits* tLibrary::Get(int objid) { 
//   if (objid <= size) { return shelf[objid]; }
//   else {return NULL; }
  return shelf[objid];
}

void tLibrary::setRanges(Grid* mygrid) {
  int i,j;
  gNode* item;
  Fits* thisone;
  for (i=0;i<mygrid->xsize;i++)
    for (j=0; j<mygrid->ysize;j++) {
      item = mygrid->grid[j][i]->Top();
      while (item != NULL) {
	//	thisone = shelf[item->Num()];
	item->setRange(shelf[item->Num()]);
        item->setISOFlux(shelf[item->Num()]->totflux);
        item = mygrid->grid[j][i]->Next(item);
      }
    }
}

