/* This class contains a library of template FITS images, indexed by object number.
A "smart" library has a certain buffer, and when an object is requested, it checks
to see if it's been read in already, and will go get it from disk if necessary. An
"unsmart" library assumes that the clever patron only asks for images that are on
the shelf, and returns an empty FITS image (?) if the object doesn't exist. */


//#include "fits.h"
//#include "structs.h"

#include <map>  //- don't do this in header, but in .cc file
using std::map;

#ifndef TLIBRARY_H
#define TLIBRARY_H
class tLibrary {
  friend class Fits;
 private:
  char *title;
  int size;
  map<int, Fits *> shelf;
  bool smart;

public:
  tLibrary();
  tLibrary(char* directory);
  tLibrary(Grid* mygrid, char* directory, float crpix1, float crpix2);
  tLibrary(char* directory, int size);
  ~tLibrary();

  Fits* Get(int objid);
  void optimize(int, int);
  void setRanges(Grid*);

  int fluxnorm;
};
#endif
